'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /* global window */

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

var _cspaceRefname = require('cspace-refname');

var _LineInput = require('./LineInput');

var _LineInput2 = _interopRequireDefault(_LineInput);

var _FilteringDropdownMenuInput = require('./FilteringDropdownMenuInput');

var _FilteringDropdownMenuInput2 = _interopRequireDefault(_FilteringDropdownMenuInput);

var _QuickAdd = require('./QuickAdd');

var _QuickAdd2 = _interopRequireDefault(_QuickAdd);

var _parseResourceID = require('../helpers/parseResourceID');

var _parseResourceID2 = _interopRequireDefault(_parseResourceID);

var _pathHelpers = require('../helpers/pathHelpers');

var _AutocompleteInput = require('../../styles/cspace-input/AutocompleteInput.css');

var _AutocompleteInput2 = _interopRequireDefault(_AutocompleteInput);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

const propTypes = _extends({}, _FilteringDropdownMenuInput2.default.propTypes, {
  findDelay: _propTypes2.default.number,
  matches: _propTypes2.default.object,
  minLength: _propTypes2.default.number,
  recordTypes: _propTypes2.default.object,
  disableAltTerms: _propTypes2.default.bool,
  showQuickAdd: _propTypes2.default.bool,
  showQuickAddCloneOption: _propTypes2.default.bool,
  quickAddCloneOptionDisabled: _propTypes2.default.bool,
  readOnly: _propTypes2.default.bool,
  asText: _propTypes2.default.bool,
  source: _propTypes2.default.string,
  quickAddTo: _propTypes2.default.string,
  addTerm: _propTypes2.default.func,
  findMatchingTerms: _propTypes2.default.func,
  formatAddPrompt: _propTypes2.default.func,
  formatCloneOptionLabel: _propTypes2.default.func,
  formatCreateNewOptionLabel: _propTypes2.default.func,
  formatMoreCharsRequiredMessage: _propTypes2.default.func,
  formatSearchResultMessage: _propTypes2.default.func,
  formatSourceName: _propTypes2.default.func,
  matchFilter: _propTypes2.default.func
});

const defaultProps = {
  findDelay: 500, // ms
  matchFilter: () => true,
  minLength: 3,
  showQuickAdd: true,
  formatMoreCharsRequiredMessage: () => 'Continue typing to find matching terms',
  formatSearchResultMessage: count => {
    const matches = count === 1 ? 'matching term' : 'matching terms';
    const num = count === 0 ? 'No' : count;

    return `${num} ${matches} found`;
  }
};

const getOptions = (partialTerm, props) => {
  const recordTypes = props.recordTypes,
        matches = props.matches,
        matchFilter = props.matchFilter,
        disableAltTerms = props.disableAltTerms,
        sourceID = props.source;


  const sources = (0, _parseResourceID2.default)(sourceID);
  const options = [];

  if (matches) {
    const partialTermMatch = matches.get(partialTerm);

    if (partialTermMatch) {
      sources.forEach(source => {
        const recordType = source.recordType,
              vocabulary = source.vocabulary;


        const sourceMatch = partialTermMatch.getIn([recordType, vocabulary]);

        if (sourceMatch) {
          const vocabDisableAltTerms = (0, _get2.default)(recordTypes, [recordType, 'vocabularies', vocabulary, 'disableAltTerms']);
          const items = sourceMatch.get('items');

          if (items) {
            items.filter(matchFilter).forEach(item => {
              // TODO: Use a prop to specify the name of the item's display name field.
              let displayNames = item.termDisplayName || item.objectNumber;

              const workflowState = item.workflowState;


              const deprecated = workflowState && workflowState.includes('deprecated');

              if (!Array.isArray(displayNames)) {
                displayNames = [displayNames];
              }

              displayNames.forEach((displayName, index) => {
                const disabled = deprecated || index > 0 && (disableAltTerms || vocabDisableAltTerms);

                options.push({
                  disabled,
                  value: (0, _cspaceRefname.setDisplayName)(item.refName, displayName),
                  label: displayName,
                  meta: item,
                  indent: index === 0 ? 0 : 1
                });
              });
            });
          }
        }
      });
    }
  }

  return options;
};

const isPending = (sourceID, matches, partialTerm) => {
  const sources = (0, _parseResourceID2.default)(sourceID);
  let foundPending = false;

  if (matches) {
    const partialTermMatch = matches.get(partialTerm);

    if (partialTermMatch) {
      sources.forEach(source => {
        const recordType = source.recordType,
              vocabulary = source.vocabulary;


        const sourceMatch = partialTermMatch.getIn([recordType, vocabulary]);

        if (sourceMatch) {
          foundPending = foundPending || sourceMatch.get('isSearchPending') || sourceMatch.get('isAddPending');
        }
      });
    }
  }

  return foundPending;
};

const getNewTerm = (sourceID, matches, partialTerm) => {
  const sources = (0, _parseResourceID2.default)(sourceID);

  let newTerm = null;

  if (matches) {
    const partialTermMatch = matches.get(partialTerm);

    if (partialTermMatch) {
      sources.forEach(source => {
        const recordType = source.recordType,
              vocabulary = source.vocabulary;


        const sourceMatch = partialTermMatch.getIn([recordType, vocabulary]);

        if (sourceMatch) {
          newTerm = newTerm || sourceMatch.get('newTerm');
        }
      });
    }
  }

  return newTerm;
};

const removeAnchorOperators = partialTerm => {
  if (!partialTerm) {
    return partialTerm;
  }

  return partialTerm.replace(/^\^/, '').replace(/\^$/, '');
};

const removeWildcardOperators = partialTerm => {
  if (!partialTerm) {
    return partialTerm;
  }

  return partialTerm.replace(/[*%_]/g, '');
};

/**
 * Remove anchor and wildcard operators from partial terms.
 */
const removePartialTermOperators = partialTerm => removeAnchorOperators(removeWildcardOperators(partialTerm));

class AutocompleteInput extends _react.Component {
  constructor(props) {
    super(props);

    this.findMatchingTerms = this.findMatchingTerms.bind(this);
    this.handleDropdownInputCommit = this.handleDropdownInputCommit.bind(this);
    this.handleFilteringDropdownMenuInputRef = this.handleFilteringDropdownMenuInputRef.bind(this);
    this.handleQuickAddBeforeItemFocusChange = this.handleQuickAddBeforeItemFocusChange.bind(this);
    this.handleDropdownBeforeItemFocusChange = this.handleDropdownBeforeItemFocusChange.bind(this);
    this.handleQuickAddRef = this.handleQuickAddRef.bind(this);
    this.handleDropdownMenuInputRef = this.handleDropdownMenuInputRef.bind(this);
    this.handleFocusPopup = this.handleFocusPopup.bind(this);

    this.state = {
      partialTerm: null,
      value: props.value,
      isFindTimerActive: false
    };
  }

  componentWillReceiveProps(nextProps) {
    const newTerm = getNewTerm(nextProps.source, nextProps.matches, this.state.partialTerm);

    const hadNewTerm = getNewTerm(this.props.source, this.props.matches, this.state.partialTerm);

    if (newTerm && !hadNewTerm) {
      const refName = newTerm.getIn(['document', 'ns2:collectionspace_core', 'refName']);
      const uri = newTerm.getIn(['document', 'ns2:collectionspace_core', 'uri']);
      const csid = uri.substring(uri.lastIndexOf('/') + 1);

      this.commit(refName, { csid });
      this.filteringDropdownMenuInput.close();
    } else {
      const newState = {
        value: nextProps.value
      };

      if (!isPending(nextProps.source, nextProps.matches, this.state.partialTerm)) {
        newState.options = getOptions(this.state.partialTerm, nextProps);
      }

      this.setState(newState);
    }
  }

  commit(value, meta) {
    this.setState({
      options: [],
      partialTerm: null,
      value
    });

    const onCommit = this.props.onCommit;


    const csid = meta ? meta.csid : undefined;

    if (onCommit) {
      onCommit((0, _pathHelpers.getPath)(this.props), value, csid);
    }
  }

  findMatchingTerms(partialTerm) {
    if (this.findTimer) {
      window.clearTimeout(this.findTimer);

      this.findTimer = null;
    }

    var _props = this.props;
    const findDelay = _props.findDelay,
          findMatchingTerms = _props.findMatchingTerms,
          matches = _props.matches,
          minLength = _props.minLength,
          source = _props.source;


    const newState = {
      partialTerm
    };

    const searchNeeded = findMatchingTerms && partialTerm && removePartialTermOperators(partialTerm).length >= minLength && (!matches || !matches.has(partialTerm));

    if (searchNeeded) {
      this.findTimer = window.setTimeout(() => {
        findMatchingTerms(source, partialTerm);

        this.findTimer = null;

        this.setState({
          isFindTimerActive: false
        });
      }, findDelay);
    } else {
      newState.options = getOptions(partialTerm, this.props);
    }

    newState.isFindTimerActive = !!this.findTimer;

    this.setState(newState);
  }

  handleDropdownInputCommit(path, value, meta) {
    this.commit(value, meta);
  }

  handleFilteringDropdownMenuInputRef(ref) {
    this.filteringDropdownMenuInput = ref;
  }

  handleDropdownMenuInputRef(ref) {
    this.dropdownMenuInput = ref;
  }

  handleQuickAddRef(ref) {
    this.quickAdd = ref;
  }

  handleQuickAddBeforeItemFocusChange(currentFocusedIndex, nextFocusedIndex, eventKey) {
    const options = this.state.options;


    if (this.dropdownMenuInput && options && options.length > 0) {
      if (nextFocusedIndex === 0 && eventKey === 'ArrowDown') {
        this.dropdownMenuInput.focusMenu(0);
        return null;
      } else if (currentFocusedIndex <= 0 && eventKey === 'ArrowUp') {
        this.dropdownMenuInput.focusMenu(-1);
        return null;
      }
    }

    return nextFocusedIndex;
  }

  handleDropdownBeforeItemFocusChange(currentFocusedIndex, nextFocusedIndex, eventKey) {
    if (this.quickAdd) {
      if (nextFocusedIndex === 0 && eventKey === 'ArrowDown') {
        this.quickAdd.focusMenu(0);
        return null;
      } else if (currentFocusedIndex <= 0 && eventKey === 'ArrowUp') {
        this.quickAdd.focusMenu(-1);
        return null;
      }
    }

    return nextFocusedIndex;
  }

  handleFocusPopup() {
    const options = this.state.options;


    if (options && options.length > 0) {
      if (this.dropdownMenuInput) {
        this.dropdownMenuInput.focusMenu(0);
      }
    } else if (this.quickAdd) {
      this.quickAdd.focusMenu(0);
    }
  }

  renderMenuFooter() {
    const menuFooter = this.props.menuFooter;


    return _react2.default.createElement(
      'div',
      null,
      this.renderQuickAdd(),
      menuFooter
    );
  }

  renderQuickAdd() {
    var _props2 = this.props;
    const addTerm = _props2.addTerm,
          formatAddPrompt = _props2.formatAddPrompt,
          formatCloneOptionLabel = _props2.formatCloneOptionLabel,
          formatCreateNewOptionLabel = _props2.formatCreateNewOptionLabel,
          formatSourceName = _props2.formatSourceName,
          minLength = _props2.minLength,
          quickAddTo = _props2.quickAddTo,
          recordTypes = _props2.recordTypes,
          showQuickAdd = _props2.showQuickAdd,
          showQuickAddCloneOption = _props2.showQuickAddCloneOption,
          quickAddCloneOptionDisabled = _props2.quickAddCloneOptionDisabled,
          source = _props2.source;


    const to = typeof quickAddTo === 'undefined' ? source : quickAddTo;

    const partialTerm = this.state.partialTerm;


    const partialTermText = removeAnchorOperators(partialTerm);

    if (showQuickAdd && partialTerm && partialTermText.length >= minLength) {
      return _react2.default.createElement(_QuickAdd2.default, {
        add: addTerm,
        displayName: partialTermText,
        partialTerm: partialTerm,
        formatAddPrompt: formatAddPrompt,
        formatCloneOptionLabel: formatCloneOptionLabel,
        formatCreateNewOptionLabel: formatCreateNewOptionLabel,
        formatDestinationName: formatSourceName,
        recordTypes: recordTypes,
        showCloneOption: showQuickAddCloneOption,
        cloneOptionDisabled: quickAddCloneOptionDisabled,
        to: to,
        onBeforeItemFocusChange: this.handleQuickAddBeforeItemFocusChange,
        ref: this.handleQuickAddRef
      });
    }

    return null;
  }

  renderReadOnly() {
    var _props3 = this.props;
    const asText = _props3.asText,
          embedded = _props3.embedded,
          readOnly = _props3.readOnly;
    const value = this.state.value;


    return _react2.default.createElement(_LineInput2.default, {
      asText: asText,
      readOnly: readOnly,
      value: (0, _cspaceRefname.getDisplayName)(value),
      embedded: embedded
    });
  }

  render() {
    var _props4 = this.props;

    const formatMoreCharsRequiredMessage = _props4.formatMoreCharsRequiredMessage,
          formatSearchResultMessage = _props4.formatSearchResultMessage,
          matches = _props4.matches,
          minLength = _props4.minLength,
          readOnly = _props4.readOnly,
          asText = _props4.asText,
          source = _props4.source,
          addTerm = _props4.addTerm,
          findDelay = _props4.findDelay,
          findMatchingTerms = _props4.findMatchingTerms,
          formatAddPrompt = _props4.formatAddPrompt,
          formatCloneOptionLabel = _props4.formatCloneOptionLabel,
          formatCreateNewOptionLabel = _props4.formatCreateNewOptionLabel,
          formatSourceName = _props4.formatSourceName,
          matchFilter = _props4.matchFilter,
          recordTypes = _props4.recordTypes,
          quickAddTo = _props4.quickAddTo,
          showQuickAdd = _props4.showQuickAdd,
          showQuickAddCloneOption = _props4.showQuickAddCloneOption,
          quickAddCloneOptionDisabled = _props4.quickAddCloneOptionDisabled,
          remainingProps = _objectWithoutProperties(_props4, ['formatMoreCharsRequiredMessage', 'formatSearchResultMessage', 'matches', 'minLength', 'readOnly', 'asText', 'source', 'addTerm', 'findDelay', 'findMatchingTerms', 'formatAddPrompt', 'formatCloneOptionLabel', 'formatCreateNewOptionLabel', 'formatSourceName', 'matchFilter', 'recordTypes', 'quickAddTo', 'showQuickAdd', 'showQuickAddCloneOption', 'quickAddCloneOptionDisabled']);

    var _state = this.state;
    const isFindTimerActive = _state.isFindTimerActive,
          options = _state.options,
          partialTerm = _state.partialTerm,
          value = _state.value;


    if (asText || readOnly) {
      return this.renderReadOnly();
    }

    const moreCharsRequired = typeof partialTerm !== 'undefined' && partialTerm !== null && removePartialTermOperators(partialTerm).length < minLength;

    const formatStatusMessage = moreCharsRequired ? formatMoreCharsRequiredMessage : formatSearchResultMessage;

    const className = isFindTimerActive || isPending(source, matches, partialTerm) ? _AutocompleteInput2.default.searching : _AutocompleteInput2.default.normal;

    return _react2.default.createElement(_FilteringDropdownMenuInput2.default, _extends({}, remainingProps, {
      className: className,
      filter: this.findMatchingTerms,
      formatStatusMessage: formatStatusMessage,
      menuFooter: this.renderMenuFooter(),
      openOnMouseDown: false,
      options: options,
      ref: this.handleFilteringDropdownMenuInputRef,
      value: value,
      valueLabel: (0, _cspaceRefname.getDisplayName)(value) || value,
      onCommit: this.handleDropdownInputCommit,
      onBeforeItemFocusChange: this.handleDropdownBeforeItemFocusChange,
      onMount: this.handleDropdownMenuInputRef,
      focusPopup: this.handleFocusPopup
    }));
  }
}

exports.default = AutocompleteInput;
AutocompleteInput.propTypes = propTypes;
AutocompleteInput.defaultProps = defaultProps;