'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _DropdownMenuInput = require('./DropdownMenuInput');

var _DropdownMenuInput2 = _interopRequireDefault(_DropdownMenuInput);

var _changeable = require('../enhancers/changeable');

var _changeable2 = _interopRequireDefault(_changeable);

var _optionHelpers = require('../helpers/optionHelpers');

var _pathHelpers = require('../helpers/pathHelpers');

var _ComboBoxInput = require('../../styles/cspace-input/ComboBoxInput.css');

var _ComboBoxInput2 = _interopRequireDefault(_ComboBoxInput);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

const DropdownMenuInput = (0, _changeable2.default)(_DropdownMenuInput2.default);

const propTypes = _extends({}, DropdownMenuInput.propTypes, {
  blankable: _propTypes2.default.bool,
  onAddOption: _propTypes2.default.func
});

const defaultProps = {
  blankable: true
};

class ComboBoxInput extends _react.Component {
  constructor(props) {
    super(props);

    this.handleDropdownInputChange = this.handleDropdownInputChange.bind(this);
    this.handleDropdownInputClose = this.handleDropdownInputClose.bind(this);
    this.handleDropdownInputCommit = this.handleDropdownInputCommit.bind(this);
    this.handleDropdownInputKeyDown = this.handleDropdownInputKeyDown.bind(this);
    this.handleDropdownInputOpen = this.handleDropdownInputOpen.bind(this);

    this.state = {
      isAdding: false,
      open: false,
      value: this.props.value
    };
  }

  componentWillReceiveProps(nextProps) {
    this.setState({
      value: nextProps.value
    });
  }

  commit(value) {
    const onCommit = this.props.onCommit;


    if (onCommit) {
      onCommit((0, _pathHelpers.getPath)(this.props), value);
    }
  }

  handleDropdownInputChange(value) {
    this.setState({
      isAdding: true,
      open: true,
      valueLabel: value
    });
  }

  handleDropdownInputCommit(path, value) {
    this.setState({
      value,
      isAdding: false,
      open: false
    });

    this.commit(value);
  }

  handleDropdownInputKeyDown(event) {
    if (event.key === 'Enter') {
      var _state = this.state;
      const isAdding = _state.isAdding,
            valueLabel = _state.valueLabel;


      if (isAdding) {
        var _props = this.props;
        const blankable = _props.blankable,
              options = _props.options,
              onAddOption = _props.onAddOption;

        // If the current content of the input matches an option exactly, select it.

        const matchingOption = (0, _optionHelpers.getOptionForLabel)(options, valueLabel);

        if (matchingOption) {
          this.setState({
            isAdding: false,
            open: false,
            value: matchingOption.value,
            valueLabel: matchingOption.valueLabel
          });

          this.commit(matchingOption.value);
        } else if (valueLabel !== '' || blankable) {
          this.setState({
            valueLabel,
            open: false
          });

          if (onAddOption) {
            onAddOption(valueLabel);
          }
        }
      }
    }
  }

  handleDropdownInputClose() {
    this.setState({
      isAdding: false,
      open: false
    });

    const onClose = this.props.onClose;


    if (onClose) {
      onClose();
    }
  }

  handleDropdownInputOpen() {
    this.setState({
      open: true
    });
  }

  render() {
    var _state2 = this.state;
    const isAdding = _state2.isAdding,
          open = _state2.open,
          value = _state2.value,
          valueLabel = _state2.valueLabel;
    var _props2 = this.props;

    const className = _props2.className,
          onAddOption = _props2.onAddOption,
          remainingProps = _objectWithoutProperties(_props2, ['className', 'onAddOption']);

    let valueProp;

    if (isAdding) {
      valueProp = { valueLabel };
    } else if ((0, _optionHelpers.getOptionForValue)(this.props.options, value)) {
      valueProp = { value };
    } else {
      valueProp = { valueLabel: value };
    }

    const classes = (0, _classnames2.default)(className, {
      [_ComboBoxInput2.default.adding]: isAdding
    });

    return _react2.default.createElement(DropdownMenuInput, _extends({}, remainingProps, valueProp, {
      className: classes,
      open: open,
      onChange: this.handleDropdownInputChange,
      onClose: this.handleDropdownInputClose,
      onCommit: this.handleDropdownInputCommit,
      onKeyDown: this.handleDropdownInputKeyDown,
      onOpen: this.handleDropdownInputOpen
    }));
  }
}

exports.default = ComboBoxInput;
ComboBoxInput.propTypes = propTypes;
ComboBoxInput.defaultProps = defaultProps;