'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

var _pathHelpers = require('../helpers/pathHelpers');

var _inputHelpers = require('../helpers/inputHelpers');

var _CompoundInput = require('../../styles/cspace-input/CompoundInput.css');

var _CompoundInput2 = _interopRequireDefault(_CompoundInput);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const propTypes = {
  children: _propTypes2.default.node,
  className: _propTypes2.default.string,
  defaultChildSubpath: _pathHelpers.pathPropType,
  name: _propTypes2.default.string,
  parentPath: _pathHelpers.pathPropType, // eslint-disable-line react/no-unused-prop-types
  subpath: _pathHelpers.pathPropType, // eslint-disable-line react/no-unused-prop-types
  readOnly: _propTypes2.default.bool,
  value: _propTypes2.default.oneOfType([_propTypes2.default.object, // eslint-disable-line react/forbid-prop-types
  _propTypes2.default.instanceOf(_immutable2.default.Map)])
};

const defaultProps = {
  value: {}
};

const getChildValue = (value, subpath, name) => {
  let keyPath = [];

  if (subpath) {
    keyPath = keyPath.concat(subpath);
  }

  if (name) {
    keyPath = keyPath.concat(name);
  }

  if (keyPath.length === 0) {
    return value;
  }

  if (_immutable2.default.Map.isMap(value)) {
    return value.getIn(keyPath);
  }

  return (0, _get2.default)(value, keyPath);
};

class CustomCompoundInput extends _react.Component {
  decorateInputs(children) {
    const readOnly = this.props.readOnly;


    return _react2.default.Children.map(children, child => {
      if (!child || !child.type) {
        // Undefined/null child or text node. Just return it.
        return child;
      }

      if ((0, _inputHelpers.isInput)(child)) {
        const name = child.props.name;
        let subpath = child.props.subpath;
        var _props = this.props;
        const defaultChildSubpath = _props.defaultChildSubpath,
              value = _props.value;


        if (typeof subpath === 'undefined') {
          subpath = defaultChildSubpath;
        }

        return _react2.default.cloneElement(child, {
          readOnly,
          subpath,
          parentPath: (0, _pathHelpers.getPath)(this.props),
          value: getChildValue(value, subpath, name)
        });
      }

      return _react2.default.cloneElement(child, {
        children: this.decorateInputs(child.props.children)
      });
    }, this);
  }

  render() {
    var _props2 = this.props;
    const children = _props2.children,
          className = _props2.className,
          name = _props2.name,
          readOnly = _props2.readOnly;


    const classes = (0, _classnames2.default)(className, _CompoundInput2.default.common, {
      [_CompoundInput2.default.readOnly]: readOnly
    });

    return _react2.default.createElement(
      'fieldset',
      {
        className: classes,
        name: name
      },
      this.decorateInputs(children)
    );
  }
}

exports.default = CustomCompoundInput;
CustomCompoundInput.propTypes = propTypes;
CustomCompoundInput.defaultProps = defaultProps;