'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _Popup = require('cspace-layout/lib/components/Popup');

var _Popup2 = _interopRequireDefault(_Popup);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _TextInput = require('./TextInput');

var _TextInput2 = _interopRequireDefault(_TextInput);

var _DropdownInput = require('../../styles/cspace-input/DropdownInput.css');

var _DropdownInput2 = _interopRequireDefault(_DropdownInput);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

const propTypes = _extends({}, _TextInput2.default.propTypes, {
  children: _propTypes2.default.node,
  className: _propTypes2.default.string,
  embedded: _propTypes2.default.bool,
  open: _propTypes2.default.bool,
  openOnFocus: _propTypes2.default.bool,
  openOnMouseDown: _propTypes2.default.bool,
  isOpenableWhenReadOnly: _propTypes2.default.bool,
  api: _propTypes2.default.func,
  /*
   * A function that may be called to give focus to the contents of the popup. This is supplied as
   * a prop because DropdownInput does not know the contents of the popup. Only the caller knows
   * which element in the popup should receive focus.
   */
  focusPopup: _propTypes2.default.func,

  onBlur: _propTypes2.default.func,
  onBeforeClose: _propTypes2.default.func,
  onClose: _propTypes2.default.func,
  onKeyDown: _propTypes2.default.func,
  onMount: _propTypes2.default.func,
  onOpen: _propTypes2.default.func
});

const defaultProps = {
  openOnMouseDown: true
};

class DropdownInput extends _react.Component {
  constructor(props) {
    super(props);

    this.handleInputBlur = this.handleInputBlur.bind(this);
    this.handleInputFocus = this.handleInputFocus.bind(this);
    this.handleInputMouseDown = this.handleInputMouseDown.bind(this);
    this.handleInputKeyDown = this.handleInputKeyDown.bind(this);
    this.handlePopupBlur = this.handlePopupBlur.bind(this);
    this.handlePopupKeyDown = this.handlePopupKeyDown.bind(this);
    this.handlePopupMount = this.handlePopupMount.bind(this);
    this.handleRef = this.handleRef.bind(this);

    this.state = {
      open: props.open
    };
  }

  componentDidMount() {
    var _props = this.props;
    const api = _props.api,
          onMount = _props.onMount;


    if (api) {
      api({
        close: this.close.bind(this)
      });
    }

    if (onMount) {
      onMount({
        focusInput: this.focusInput.bind(this)
      });
    }
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.open) {
      this.open();
    } else {
      this.asyncClose();
    }
  }

  componentDidUpdate(prevProps, prevState) {
    if (prevState.open && !this.state.open) {
      const onClose = this.props.onClose;


      if (onClose) {
        onClose();
      }
    } else if (!prevState.open && this.state.open) {
      const onOpen = this.props.onOpen;


      if (onOpen) {
        onOpen();
      }
    }
  }

  close(isCancelled) {
    if (this.state.open) {
      const onBeforeClose = this.props.onBeforeClose;


      if (onBeforeClose) {
        onBeforeClose(isCancelled);
      }

      this.setState({
        open: false
      });
    }
  }

  asyncClose(isCancelled) {
    setTimeout(() => {
      this.close(isCancelled);
    }, 0);
  }

  open() {
    if (!this.state.open) {
      this.setState({
        open: true
      });
    }
  }

  focusInput() {
    // FIXME: This breaks the TextInput component abstraction by selecting the rendered DOM node
    // directly and calling focus() on it. But fixing this would require saving a ref to the
    // TextInput; making TextInput a class instead of a function; and adding a focus() method to
    // that class, which could be called here. LineInput and MultilineInput would also need to
    // become classes instead of functions, each with their own focus() method, in order to
    // properly implement TextInput.focus(). This seems like overkill at the moment.

    this.domNode.querySelector('input, textarea').focus();
  }

  focusPopup() {
    const focusPopup = this.props.focusPopup;


    if (focusPopup) {
      focusPopup();
    }
  }

  handleInputBlur(event) {
    const onBlur = this.props.onBlur;


    if (onBlur) {
      onBlur(event);
    }

    if (!this.domNode.contains(event.relatedTarget)) {
      this.asyncClose();
    }
  }

  handleInputFocus() {
    const openOnFocus = this.props.openOnFocus;


    if (openOnFocus) {
      this.open();
    }
  }

  handleInputMouseDown() {
    const openOnMouseDown = this.props.openOnMouseDown;


    if (openOnMouseDown) {
      this.open();
    }
  }

  handleInputKeyDown(event) {
    const onKeyDown = this.props.onKeyDown;


    if (event.key === 'ArrowDown') {
      // Prevent the page from scrolling.
      event.preventDefault();

      if (this.state.open) {
        this.focusPopup();
      } else {
        this.open();
        this.focusPopupNeeded = true;
      }
    } else if (event.key === 'Escape') {
      this.asyncClose(true);
    }

    if (onKeyDown) {
      onKeyDown(event);
    }
  }

  handlePopupBlur(event) {
    if (!this.domNode.contains(event.relatedTarget)) {
      this.asyncClose();
    }
  }

  handlePopupKeyDown(event) {
    if (event.key === 'Escape') {
      this.asyncClose();
      this.focusInput();
    }
  }

  handlePopupMount() {
    if (this.focusPopupNeeded) {
      this.focusPopup();
      this.focusPopupNeeded = false;
    }
  }

  handleRef(ref) {
    this.domNode = ref;
  }

  renderInput() {
    var _props2 = this.props;

    const children = _props2.children,
          className = _props2.className,
          focusPopup = _props2.focusPopup,
          isOpenableWhenReadOnly = _props2.isOpenableWhenReadOnly,
          openOnFocus = _props2.openOnFocus,
          openOnMouseDown = _props2.openOnMouseDown,
          onBeforeClose = _props2.onBeforeClose,
          onClose = _props2.onClose,
          onKeyDown = _props2.onKeyDown,
          onMount = _props2.onMount,
          onOpen = _props2.onOpen,
          remainingProps = _objectWithoutProperties(_props2, ['children', 'className', 'focusPopup', 'isOpenableWhenReadOnly', 'openOnFocus', 'openOnMouseDown', 'onBeforeClose', 'onClose', 'onKeyDown', 'onMount', 'onOpen']);

    return _react2.default.createElement(_TextInput2.default, _extends({}, remainingProps, {
      onBlur: this.handleInputBlur,
      onFocus: this.handleInputFocus,
      onKeyDown: this.handleInputKeyDown,
      onMouseDown: this.handleInputMouseDown
    }));
  }

  renderDropdown() {
    const open = this.state.open;
    const children = this.props.children;


    if (open) {
      return _react2.default.createElement(
        _Popup2.default,
        {
          onBlur: this.handlePopupBlur,
          onKeyDown: this.handlePopupKeyDown,
          onMount: this.handlePopupMount
        },
        children
      );
    }

    return null;
  }

  render() {
    const open = this.state.open;
    var _props3 = this.props;
    const className = _props3.className,
          embedded = _props3.embedded,
          readOnly = _props3.readOnly,
          isOpenableWhenReadOnly = _props3.isOpenableWhenReadOnly;


    const classes = (0, _classnames2.default)(className, {
      [_DropdownInput2.default.normal]: !embedded,
      [_DropdownInput2.default.embedded]: embedded,
      [_DropdownInput2.default.open]: open
    });

    let readOnlyProps;

    if (readOnly && isOpenableWhenReadOnly) {
      readOnlyProps = {
        tabIndex: 0,
        onBlur: this.handleInputBlur,
        onFocus: this.handleInputFocus,
        onKeyDown: this.handleInputKeyDown,
        onMouseDown: this.handleInputMouseDown
      };
    }

    return _react2.default.createElement(
      'div',
      _extends({
        ref: this.handleRef,
        className: classes
      }, readOnlyProps),
      this.renderInput(),
      this.renderDropdown()
    );
  }
}

exports.default = DropdownInput;
DropdownInput.propTypes = propTypes;
DropdownInput.defaultProps = defaultProps;