'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _LineInput = require('./LineInput');

var _LineInput2 = _interopRequireDefault(_LineInput);

var _DropdownInput = require('./DropdownInput');

var _DropdownInput2 = _interopRequireDefault(_DropdownInput);

var _Menu = require('./Menu');

var _Menu2 = _interopRequireDefault(_Menu);

var _pathHelpers = require('../helpers/pathHelpers');

var _optionHelpers = require('../helpers/optionHelpers');

var _DropdownMenuInput = require('../../styles/cspace-input/DropdownMenuInput.css');

var _DropdownMenuInput2 = _interopRequireDefault(_DropdownMenuInput);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

const propTypes = _extends({}, _DropdownInput2.default.propTypes, {
  className: _propTypes2.default.string,
  menuHeader: _propTypes2.default.node,
  menuFooter: _propTypes2.default.node,
  open: _propTypes2.default.bool,
  options: _propTypes2.default.arrayOf(_propTypes2.default.shape({
    value: _propTypes2.default.string,
    label: _propTypes2.default.string
  })),
  ignoreDisabledOptions: _propTypes2.default.bool,
  readOnly: _propTypes2.default.bool,
  renderItemLabel: _propTypes2.default.func,
  valueLabel: _propTypes2.default.string,
  onClose: _propTypes2.default.func,
  onCommit: _propTypes2.default.func,
  onItemMouseEnter: _propTypes2.default.func,
  onItemMouseLeave: _propTypes2.default.func,
  onMount: _propTypes2.default.func,
  onOpen: _propTypes2.default.func,
  onUpdate: _propTypes2.default.func
});

const defaultProps = {
  blankable: true,
  options: []
};

const renderMenuHeader = content => {
  if (content) {
    return _react2.default.createElement(
      'header',
      null,
      content
    );
  }

  return null;
};

const renderMenuFooter = content => {
  if (content) {
    return _react2.default.createElement(
      'footer',
      null,
      content
    );
  }

  return null;
};

class DropdownMenuInput extends _react.Component {
  constructor(props) {
    super(props);

    this.handleDropdownInputClose = this.handleDropdownInputClose.bind(this);
    this.handleDropdownInputMount = this.handleDropdownInputMount.bind(this);
    this.handleDropdownInputOpen = this.handleDropdownInputOpen.bind(this);
    this.handleMenuRef = this.handleMenuRef.bind(this);
    this.handleMenuSelect = this.handleMenuSelect.bind(this);
    this.focusMenu = this.focusMenu.bind(this);

    const valueLabel = props.valueLabel === null || typeof props.valueLabel === 'undefined' ? (0, _optionHelpers.getLabelForValue)(props.options, props.value) : props.valueLabel;

    this.state = {
      valueLabel,
      open: false,
      value: props.value
    };
  }

  componentDidMount() {
    const onMount = this.props.onMount;


    if (onMount) {
      onMount({
        value: this.state.value,
        focusMenu: this.focusMenu.bind(this)
      });
    }
  }

  componentWillReceiveProps(nextProps) {
    const valueLabel = nextProps.valueLabel === null || typeof nextProps.valueLabel === 'undefined' ? (0, _optionHelpers.getLabelForValue)(nextProps.options, nextProps.value) : nextProps.valueLabel;

    this.setState({
      valueLabel,
      open: nextProps.open,
      value: nextProps.value
    });
  }

  componentDidUpdate() {
    const onUpdate = this.props.onUpdate;


    if (onUpdate) {
      onUpdate({ value: this.state.value });
    }
  }

  commit(value, meta) {
    var _props = this.props;
    const initialValue = _props.value,
          onCommit = _props.onCommit;


    if (onCommit && (value || initialValue) && value !== initialValue) {
      onCommit((0, _pathHelpers.getPath)(this.props), value, meta);
    }
  }

  focusMenu(itemIndex) {
    if (this.menu) {
      this.menu.focus(itemIndex);
    }
  }

  handleDropdownInputClose() {
    this.setState({
      open: false
    });

    const onClose = this.props.onClose;


    if (onClose) {
      onClose();
    }
  }

  handleDropdownInputMount({ focusInput }) {
    this.focusInput = focusInput;
  }

  handleDropdownInputOpen() {
    this.setState({
      open: true
    });

    const onOpen = this.props.onOpen;


    if (onOpen) {
      onOpen();
    }
  }

  handleMenuRef(ref) {
    this.menu = ref;
  }

  handleMenuSelect(option) {
    const value = option.value,
          meta = option.meta,
          valueLabel = option.label;


    this.setState({
      value,
      valueLabel,
      open: false
    });

    this.commit(value, meta);
    this.focusInput();
  }

  render() {
    var _state = this.state;
    const open = _state.open,
          value = _state.value,
          valueLabel = _state.valueLabel;
    var _props2 = this.props;

    const className = _props2.className,
          menuHeader = _props2.menuHeader,
          menuFooter = _props2.menuFooter,
          options = _props2.options,
          ignoreDisabledOptions = _props2.ignoreDisabledOptions,
          readOnly = _props2.readOnly,
          renderItemLabel = _props2.renderItemLabel,
          onBeforeItemFocusChange = _props2.onBeforeItemFocusChange,
          onItemMouseEnter = _props2.onItemMouseEnter,
          onItemMouseLeave = _props2.onItemMouseLeave,
          focusPopup = _props2.focusPopup,
          blankable = _props2.blankable,
          openProp = _props2.open,
          valueLabelProp = _props2.valueLabel,
          onClose = _props2.onClose,
          onCommit = _props2.onCommit,
          onMount = _props2.onMount,
          onOpen = _props2.onOpen,
          onUpdate = _props2.onUpdate,
          remainingProps = _objectWithoutProperties(_props2, ['className', 'menuHeader', 'menuFooter', 'options', 'ignoreDisabledOptions', 'readOnly', 'renderItemLabel', 'onBeforeItemFocusChange', 'onItemMouseEnter', 'onItemMouseLeave', 'focusPopup', 'blankable', 'open', 'valueLabel', 'onClose', 'onCommit', 'onMount', 'onOpen', 'onUpdate']);

    const inputValue = valueLabel;

    if (readOnly) {
      return _react2.default.createElement(_LineInput2.default, {
        readOnly: true,
        value: inputValue,
        embedded: this.props.embedded
      });
    }

    const classes = (0, _classnames2.default)(className, {
      [_DropdownMenuInput2.default.common]: true,
      [_DropdownMenuInput2.default.open]: open
    });

    return _react2.default.createElement(
      _DropdownInput2.default,
      _extends({}, remainingProps, {
        className: classes,
        open: open,
        spellCheck: false,
        value: inputValue,
        onClose: this.handleDropdownInputClose,
        onMount: this.handleDropdownInputMount,
        onOpen: this.handleDropdownInputOpen,
        focusPopup: focusPopup || this.focusMenu
      }),
      renderMenuHeader(menuHeader),
      _react2.default.createElement(_Menu2.default, {
        options: options,
        ignoreDisabledOptions: ignoreDisabledOptions,
        ref: this.handleMenuRef,
        tabIndex: '-1',
        renderItemLabel: renderItemLabel,
        value: value,
        onSelect: this.handleMenuSelect,
        onBeforeItemFocusChange: onBeforeItemFocusChange,
        onItemMouseEnter: onItemMouseEnter,
        onItemMouseLeave: onItemMouseLeave
      }),
      renderMenuFooter(menuFooter)
    );
  }
}

exports.default = DropdownMenuInput;
DropdownMenuInput.propTypes = propTypes;
DropdownMenuInput.defaultProps = defaultProps;