'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _LineInput = require('./LineInput');

var _LineInput2 = _interopRequireDefault(_LineInput);

var _pathHelpers = require('../helpers/pathHelpers');

var _FileInput = require('../../styles/cspace-input/FileInput.css');

var _FileInput2 = _interopRequireDefault(_FileInput);

var _MiniButton = require('../../styles/cspace-input/MiniButton.css');

var _MiniButton2 = _interopRequireDefault(_MiniButton);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* global File */

const fileListToArray = fileList => {
  const fileArray = [];

  for (let i = 0; i < fileList.length; i += 1) {
    fileArray.push(fileList[i]);
  }

  return fileArray;
};

const propTypes = {
  chooseButtonLabel: _propTypes2.default.string,
  name: _propTypes2.default.string,
  /* eslint-disable react/no-unused-prop-types */
  parentPath: _pathHelpers.pathPropType,
  subpath: _pathHelpers.pathPropType,
  /* eslint-enable react/no-unused-prop-types */
  value: _propTypes2.default.arrayOf(_propTypes2.default.instanceOf(File)),
  readOnly: _propTypes2.default.bool,
  formatFileInfo: _propTypes2.default.func,
  onCommit: _propTypes2.default.func
};

const defaultProps = {
  chooseButtonLabel: 'Select…',
  formatFileInfo: (name, type, size) => `${name} (${type}, ${size} bytes)`
};

class FileInput extends _react.Component {
  constructor(props) {
    super(props);

    this.handleChooseButtonClick = this.handleChooseButtonClick.bind(this);
    this.handleDragEnter = this.handleDragEnter.bind(this);
    this.handleDragLeave = this.handleDragLeave.bind(this);
    this.handleDragOver = this.handleDragOver.bind(this);
    this.handleDrop = this.handleDrop.bind(this);
    this.handleFileInputChange = this.handleFileInputChange.bind(this);
    this.handleFileInputRef = this.handleFileInputRef.bind(this);

    this.state = {
      isDraggedOver: false
    };
  }

  handleChooseButtonClick() {
    if (this.fileInput) {
      this.fileInput.click();
    }
  }

  handleDragEnter(event) {
    event.preventDefault();
    event.stopPropagation();

    this.setState({
      isDraggedOver: true
    });
  }

  handleDragLeave(event) {
    event.preventDefault();
    event.stopPropagation();

    this.setState({
      isDraggedOver: false
    });
  }

  // eslint-disable-next-line class-methods-use-this
  handleDragOver(event) {
    event.preventDefault();
    event.stopPropagation();
  }

  handleDrop(event) {
    const onCommit = this.props.onCommit;


    event.preventDefault();
    event.stopPropagation();

    const files = event.dataTransfer.files;

    if (files && files.length > 0 && onCommit) {
      onCommit((0, _pathHelpers.getPath)(this.props), fileListToArray(files));
    }

    this.setState({
      isDraggedOver: false
    });
  }

  handleFileInputChange(event) {
    const input = event.target;

    const onCommit = this.props.onCommit;


    if (onCommit) {
      onCommit((0, _pathHelpers.getPath)(this.props), fileListToArray(input.files));
    }
  }

  handleFileInputRef(ref) {
    this.fileInput = ref;
  }

  render() {
    var _props = this.props;
    const chooseButtonLabel = _props.chooseButtonLabel,
          name = _props.name,
          value = _props.value,
          readOnly = _props.readOnly,
          formatFileInfo = _props.formatFileInfo;


    if (readOnly) {
      return _react2.default.createElement(_LineInput2.default, { readOnly: true });
    }

    const isDraggedOver = this.state.isDraggedOver;


    let selectedFileDescription = null;

    if (value && value.length > 0) {
      const file = value[0];

      selectedFileDescription = formatFileInfo(file.name, file.type, file.size);
    }

    const className = isDraggedOver ? _FileInput2.default.dragOver : _FileInput2.default.normal;

    return (
      // Allow click/drag/drop events on the file info div.
      /* eslint-disable jsx-a11y/no-static-element-interactions */
      _react2.default.createElement(
        'div',
        {
          className: className
        },
        _react2.default.createElement('input', {
          name: name,
          ref: this.handleFileInputRef,
          tabIndex: '-1',
          type: 'file',
          onChange: this.handleFileInputChange
        }),
        _react2.default.createElement(
          'button',
          {
            className: _MiniButton2.default.common,
            type: 'button',
            onClick: this.handleChooseButtonClick
          },
          chooseButtonLabel
        ),
        _react2.default.createElement(
          'div',
          {
            onClick: this.handleChooseButtonClick,
            onDragEnter: this.handleDragEnter,
            onDragLeave: this.handleDragLeave,
            onDragOver: this.handleDragOver,
            onDrop: this.handleDrop
          },
          selectedFileDescription
        )
      )
      /* eslint-enable jsx-a11y/no-static-element-interactions */

    );
  }
}

exports.default = FileInput;
FileInput.propTypes = propTypes;
FileInput.defaultProps = defaultProps;