'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _DropdownMenuInput = require('./DropdownMenuInput');

var _DropdownMenuInput2 = _interopRequireDefault(_DropdownMenuInput);

var _changeable = require('../enhancers/changeable');

var _changeable2 = _interopRequireDefault(_changeable);

var _optionHelpers = require('../helpers/optionHelpers');

var _pathHelpers = require('../helpers/pathHelpers');

var _FilteringDropdownMenuInput = require('../../styles/cspace-input/FilteringDropdownMenuInput.css');

var _FilteringDropdownMenuInput2 = _interopRequireDefault(_FilteringDropdownMenuInput);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

const DropdownMenuInput = (0, _changeable2.default)(_DropdownMenuInput2.default);

const propTypes = _extends({}, DropdownMenuInput.propTypes, {
  blankable: _propTypes2.default.bool,
  formatStatusMessage: _propTypes2.default.func,
  filter: _propTypes2.default.func,
  onClose: _propTypes2.default.func,
  onOpen: _propTypes2.default.func,
  onCommit: _propTypes2.default.func
});

const defaultProps = {
  blankable: true,
  formatStatusMessage: count => {
    const matches = count === 1 ? 'match' : 'matches';
    const num = count === 0 ? 'No' : count;

    return `${num} ${matches} found`;
  }
};

class FilteringDropdownMenuInput extends _react.Component {
  constructor(props) {
    super(props);

    this.handleDropdownInputBeforeClose = this.handleDropdownInputBeforeClose.bind(this);
    this.handleDropdownInputChange = this.handleDropdownInputChange.bind(this);
    this.handleDropdownInputClose = this.handleDropdownInputClose.bind(this);
    this.handleDropdownInputCommit = this.handleDropdownInputCommit.bind(this);
    this.handleDropdownInputKeyDown = this.handleDropdownInputKeyDown.bind(this);
    this.handleDropdownInputOpen = this.handleDropdownInputOpen.bind(this);

    this.state = {
      isFiltering: false,
      open: false,
      value: this.props.value
    };
  }

  componentWillReceiveProps(nextProps) {
    this.setState({
      value: nextProps.value
    });
  }

  close() {
    this.setState({
      open: false
    });
  }

  commit(value, meta) {
    const onCommit = this.props.onCommit;


    if (onCommit) {
      onCommit((0, _pathHelpers.getPath)(this.props), value, meta);
    }
  }

  filter(filterByValue) {
    const filter = this.props.filter;


    if (filter) {
      filter(filterByValue);
    }
  }

  handleDropdownInputBeforeClose(isCancelled) {
    if (isCancelled) {
      this.setState({
        isFiltering: false
      });
    }
  }

  handleDropdownInputChange(value) {
    this.setState({
      isFiltering: true,
      open: true,
      valueLabel: value
    });

    this.filter(value);
  }

  handleDropdownInputCommit(path, value, meta) {
    this.setState({
      value,
      isFiltering: false,
      open: false
    });

    this.filter();
    this.commit(value, meta);
  }

  handleDropdownInputKeyDown(event) {
    const isFiltering = this.state.isFiltering;


    if (isFiltering && event.key === 'Enter') {
      event.preventDefault();

      const valueLabel = this.state.valueLabel;
      var _props = this.props;
      const blankable = _props.blankable,
            options = _props.options,
            ignoreDisabledOptions = _props.ignoreDisabledOptions;

      // If there is only one option, select it. Otherwise, if the current content of the
      // input matches an option exactly, select it.

      let matchingOption = options.length === 1 ? options[0] : (0, _optionHelpers.getOptionForLabel)(options, valueLabel);

      if (!matchingOption && blankable && valueLabel === '') {
        // If blankable, and the current content of the input is empty, allow it to be
        // committed even if there is no empty option.

        matchingOption = {
          value: '',
          valueLabel: ''
        };
      }

      // The matching option must not be disabled.

      if (matchingOption && matchingOption.disabled && !ignoreDisabledOptions) {
        matchingOption = null;
      }

      if (matchingOption) {
        this.setState({
          open: false,
          value: matchingOption.value,
          valueLabel: matchingOption.label
        });

        this.commit(matchingOption.value, matchingOption.meta);
      }
    }
  }

  handleDropdownInputClose() {
    const nextState = {
      isFiltering: false,
      open: false
    };

    var _state = this.state;
    const isFiltering = _state.isFiltering,
          valueLabel = _state.valueLabel;


    if (isFiltering && valueLabel === '') {
      // Normally enter must be pressed on the value being filtered in order to select a matching
      // option, but make an exception for the blank value. If the dropdown is closed while
      // filtering, and the filter value is blank, automatically select the corresponding option
      // if there is one, or if blankable is true. This allows fields to be cleared without ever
      // pressing enter. This is required by DRYD-227.

      var _props2 = this.props;
      const blankable = _props2.blankable,
            options = _props2.options;


      let matchingOption = (0, _optionHelpers.getOptionForLabel)(options, valueLabel);

      if (!matchingOption && blankable) {
        // If blankable, and the current content of the input is empty, allow it to be
        // committed even if there is no empty option.

        matchingOption = {
          value: '',
          valueLabel: ''
        };
      }

      if (matchingOption) {
        nextState.value = matchingOption.value;
        nextState.valueLabel = matchingOption.label;

        this.commit(matchingOption.value, matchingOption.meta);
      }
    }

    this.setState(nextState);

    const onClose = this.props.onClose;


    if (onClose) {
      onClose();
    }
  }

  handleDropdownInputOpen() {
    const onOpen = this.props.onOpen;


    if (onOpen) {
      onOpen();
    }

    if (!this.state.open) {
      this.setState({
        isFiltering: false,
        open: true
      });

      this.filter();
    }
  }

  renderMenuHeader() {
    const isFiltering = this.state.isFiltering;
    var _props3 = this.props;
    const formatStatusMessage = _props3.formatStatusMessage,
          options = _props3.options;


    if (isFiltering) {
      return _react2.default.createElement(
        'span',
        null,
        formatStatusMessage(options ? options.length : 0)
      );
    }

    return null;
  }

  render() {
    var _state2 = this.state;
    const isFiltering = _state2.isFiltering,
          open = _state2.open,
          value = _state2.value,
          valueLabel = _state2.valueLabel;
    var _props4 = this.props;

    const onMount = _props4.onMount,
          className = _props4.className,
          formatStatusMessage = _props4.formatStatusMessage,
          remainingProps = _objectWithoutProperties(_props4, ['onMount', 'className', 'formatStatusMessage']);

    const valueProp = isFiltering ? { valueLabel } : { value };

    const classes = (0, _classnames2.default)(className, {
      [_FilteringDropdownMenuInput2.default.filtering]: isFiltering
    });

    return _react2.default.createElement(DropdownMenuInput, _extends({}, remainingProps, valueProp, {
      className: classes,
      menuHeader: this.renderMenuHeader(),
      open: open,
      onBeforeClose: this.handleDropdownInputBeforeClose,
      onChange: this.handleDropdownInputChange,
      onClose: this.handleDropdownInputClose,
      onCommit: this.handleDropdownInputCommit,
      onKeyDown: this.handleDropdownInputKeyDown,
      onOpen: this.handleDropdownInputOpen,
      onMount: onMount
    }));
  }
}

exports.default = FilteringDropdownMenuInput;
FilteringDropdownMenuInput.propTypes = propTypes;
FilteringDropdownMenuInput.defaultProps = defaultProps;