'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _Row = require('cspace-layout/lib/components/Row');

var _Row2 = _interopRequireDefault(_Row);

var _DropdownInput = require('./DropdownInput');

var _DropdownInput2 = _interopRequireDefault(_DropdownInput);

var _LineInput = require('./LineInput');

var _LineInput2 = _interopRequireDefault(_LineInput);

var _Menu = require('./Menu');

var _Menu2 = _interopRequireDefault(_Menu);

var _changeable = require('../enhancers/changeable');

var _changeable2 = _interopRequireDefault(_changeable);

var _committable = require('../enhancers/committable');

var _committable2 = _interopRequireDefault(_committable);

var _pathHelpers = require('../helpers/pathHelpers');

var _IDGeneratorInput = require('../../styles/cspace-input/IDGeneratorInput.css');

var _IDGeneratorInput2 = _interopRequireDefault(_IDGeneratorInput);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

const DropdownInput = (0, _committable2.default)((0, _changeable2.default)(_DropdownInput2.default));

const propTypes = _extends({}, DropdownInput.propTypes, {
  patterns: _propTypes2.default.array,
  generateID: _propTypes2.default.func,
  sampleColumnLabel: _propTypes2.default.string,
  typeColumnLabel: _propTypes2.default.string,
  readOnly: _propTypes2.default.bool,
  onMount: _propTypes2.default.func,
  onOpen: _propTypes2.default.func
});

const defaultProps = {
  patterns: [],
  sampleColumnLabel: 'Sample',
  typeColumnLabel: 'Type'
};

const renderItemLabel = label => _react2.default.createElement(
  _Row2.default,
  null,
  _react2.default.createElement(
    'div',
    null,
    label[0]
  ),
  _react2.default.createElement(
    'div',
    null,
    label[1]
  )
);

class IDGeneratorInput extends _react.Component {
  constructor(props) {
    super(props);

    this.focusMenu = this.focusMenu.bind(this);
    this.handleDropdownInputClose = this.handleDropdownInputClose.bind(this);
    this.handleDropdownInputMount = this.handleDropdownInputMount.bind(this);
    this.handleDropdownInputOpen = this.handleDropdownInputOpen.bind(this);
    this.handleMenuRef = this.handleMenuRef.bind(this);
    this.handleMenuSelect = this.handleMenuSelect.bind(this);

    this.state = {
      open: false
    };
  }

  componentDidMount() {
    const onMount = this.props.onMount;


    if (onMount) {
      onMount();
    }
  }

  focusMenu() {
    if (this.menu) {
      this.menu.focus();
    }
  }

  handleDropdownInputMount({ focusInput }) {
    this.focusInput = focusInput;
  }

  handleDropdownInputClose() {
    this.setState({
      open: false
    });
  }

  handleDropdownInputOpen() {
    var _props = this.props;
    const onOpen = _props.onOpen,
          patterns = _props.patterns;


    if (onOpen) {
      onOpen(patterns);
    }

    this.setState({
      open: true
    });
  }

  handleMenuRef(ref) {
    this.menu = ref;
  }

  handleMenuSelect(option) {
    this.setState({
      open: false
    });

    this.focusInput();

    const generateID = this.props.generateID;


    if (generateID) {
      generateID(option.value, (0, _pathHelpers.getPath)(this.props));
    }
  }

  render() {
    const open = this.state.open;
    var _props2 = this.props;

    const patterns = _props2.patterns,
          sampleColumnLabel = _props2.sampleColumnLabel,
          typeColumnLabel = _props2.typeColumnLabel,
          readOnly = _props2.readOnly,
          generateID = _props2.generateID,
          onMount = _props2.onMount,
          onOpen = _props2.onOpen,
          remainingProps = _objectWithoutProperties(_props2, ['patterns', 'sampleColumnLabel', 'typeColumnLabel', 'readOnly', 'generateID', 'onMount', 'onOpen']);

    if (readOnly) {
      return _react2.default.createElement(_LineInput2.default, {
        readOnly: true,
        value: this.props.value,
        embedded: this.props.embedded
      });
    }

    const options = patterns.map(pattern => ({
      value: pattern.name,
      label: [pattern.type, pattern.sample]
    }));

    return _react2.default.createElement(
      DropdownInput,
      _extends({}, remainingProps, {
        className: _IDGeneratorInput2.default.normal,
        focusPopup: this.focusMenu,
        open: open,
        onClose: this.handleDropdownInputClose,
        onMount: this.handleDropdownInputMount,
        onOpen: this.handleDropdownInputOpen
      }),
      _react2.default.createElement(
        'header',
        null,
        _react2.default.createElement(
          _Row2.default,
          null,
          _react2.default.createElement(
            'div',
            null,
            typeColumnLabel
          ),
          _react2.default.createElement(
            'div',
            null,
            sampleColumnLabel
          )
        )
      ),
      _react2.default.createElement(_Menu2.default, {
        options: options,
        ref: this.handleMenuRef,
        renderItemLabel: renderItemLabel,
        tabIndex: '-1',
        onSelect: this.handleMenuSelect
      })
    );
  }
}

exports.default = IDGeneratorInput;
IDGeneratorInput.propTypes = propTypes;
IDGeneratorInput.defaultProps = defaultProps;