'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = InputTableHeader;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _inputHelpers = require('../helpers/inputHelpers');

var _normalizeLabel = require('../helpers/normalizeLabel');

var _normalizeLabel2 = _interopRequireDefault(_normalizeLabel);

var _InputTableHeader = require('../../styles/cspace-input/InputTableHeader.css');

var _InputTableHeader2 = _interopRequireDefault(_InputTableHeader);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const propTypes = {
  children: _propTypes2.default.node,
  embedded: _propTypes2.default.bool,
  renderLabel: _propTypes2.default.func,
  sortableFields: _propTypes2.default.object,
  onSortButtonClick: _propTypes2.default.func
};

const defaultProps = {
  renderLabel: input => (0, _normalizeLabel2.default)(input.props.label)
};

function InputTableHeader(props) {
  const children = props.children,
        embedded = props.embedded,
        renderLabel = props.renderLabel,
        sortableFields = props.sortableFields,
        onSortButtonClick = props.onSortButtonClick;


  const inputs = (0, _inputHelpers.extractInputs)(children);
  const labels = inputs.map(input => renderLabel(input));

  if (!labels.some(label => !!label)) {
    return null;
  }

  const labelContainers = labels.map((label, index) => {
    const input = inputs[index];

    var _input$props = input.props;
    const flex = _input$props.flex,
          name = _input$props.name;


    const style = flex ? { flex } : undefined;

    let wrappedLabel = label;

    if (onSortButtonClick && sortableFields && sortableFields[name]) {
      wrappedLabel = _react2.default.createElement(
        'button',
        { name: name, type: 'button', onClick: onSortButtonClick },
        label
      );
    }

    return _react2.default.createElement(
      'div',
      {
        key: name || index,
        style: style
      },
      wrappedLabel
    );
  });

  const className = embedded ? _InputTableHeader2.default.embedded : _InputTableHeader2.default.normal;

  return _react2.default.createElement(
    'div',
    { className: className },
    labelContainers
  );
}

InputTableHeader.propTypes = propTypes;
InputTableHeader.defaultProps = defaultProps;