'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _pathHelpers = require('../helpers/pathHelpers');

var _LineInput = require('../../styles/cspace-input/LineInput.css');

var _LineInput2 = _interopRequireDefault(_LineInput);

var _LineInputWrapper = require('../../styles/cspace-input/LineInputWrapper.css');

var _LineInputWrapper2 = _interopRequireDefault(_LineInputWrapper);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

const propTypes = {
  asText: _propTypes2.default.bool,
  embedded: _propTypes2.default.bool,
  name: _propTypes2.default.string,
  parentPath: _pathHelpers.pathPropType,
  subpath: _pathHelpers.pathPropType,
  value: _propTypes2.default.string,
  readOnly: _propTypes2.default.bool,
  showLink: _propTypes2.default.bool,
  linkContent: _propTypes2.default.oneOfType([_propTypes2.default.element, _propTypes2.default.string]),
  linkUrl: _propTypes2.default.func,
  api: _propTypes2.default.func
};

const defaultProps = {
  linkContent: 'Open',
  linkUrl: value => value && value.startsWith('http://') ? value : null
};

/**
 * A text input that accepts and is able to display only a single line of text. If a value prop is
 * supplied that contains a newline character, the behavior is unspecified; newline characters may
 * be stripped, replaced with other characters, or retained but not displayed. If this presents a
 * problem, use TextInput or MultilineInput.
 */
class LineInput extends _react.Component {
  constructor() {
    super();

    this.handleRef = this.handleRef.bind(this);
  }

  componentDidMount() {
    const api = this.props.api;


    if (api) {
      api({
        focus: this.focus.bind(this)
      });
    }
  }

  componentWillUnmount() {
    const api = this.props.api;


    if (api) {
      api(null);
    }
  }

  focus() {
    this.domNode.focus();
  }

  handleRef(ref) {
    this.domNode = ref;
  }

  render() {
    var _props = this.props;

    const asText = _props.asText,
          embedded = _props.embedded,
          readOnly = _props.readOnly,
          value = _props.value,
          showLink = _props.showLink,
          linkContent = _props.linkContent,
          linkUrl = _props.linkUrl,
          parentPath = _props.parentPath,
          subpath = _props.subpath,
          api = _props.api,
          remainingProps = _objectWithoutProperties(_props, ['asText', 'embedded', 'readOnly', 'value', 'showLink', 'linkContent', 'linkUrl', 'parentPath', 'subpath', 'api']);

    const className = embedded ? _LineInput2.default.embedded : _LineInput2.default.normal;
    const normalizedValue = value === null || typeof value === 'undefined' ? '' : value;

    if (asText) {
      return _react2.default.createElement(
        'div',
        { className: className },
        normalizedValue
      );
    }

    const input = _react2.default.createElement('input', _extends({}, remainingProps, {
      className: className,
      disabled: readOnly,
      readOnly: !remainingProps.onChange,
      ref: this.handleRef,
      type: 'text',
      value: normalizedValue
    }));

    if (showLink) {
      const url = linkUrl(value);

      const link = url ? _react2.default.createElement(
        'a',
        { href: url, target: '_blank', rel: 'noopener noreferrer' },
        linkContent
      ) : null;

      return _react2.default.createElement(
        'div',
        { className: _LineInputWrapper2.default.common },
        input,
        link
      );
    }

    return input;
  }
}

exports.default = LineInput;
LineInput.propTypes = propTypes;
LineInput.defaultProps = defaultProps;