'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _Menu = require('../../styles/cspace-input/Menu.css');

var _Menu2 = _interopRequireDefault(_Menu);

var _MenuItem = require('../../styles/cspace-input/MenuItem.css');

var _MenuItem2 = _interopRequireDefault(_MenuItem);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const propTypes = {
  options: _propTypes2.default.arrayOf(_propTypes2.default.shape({
    value: _propTypes2.default.string
  })),
  tabIndex: _propTypes2.default.string,
  renderItemLabel: _propTypes2.default.func,
  value: _propTypes2.default.string,
  ignoreDisabledOptions: _propTypes2.default.bool,
  onSelect: _propTypes2.default.func,
  onBeforeItemFocusChange: _propTypes2.default.func,
  onItemMouseEnter: _propTypes2.default.func,
  onItemMouseLeave: _propTypes2.default.func
};

const defaultProps = {
  options: [],
  // Display a no break space if the label is blank, so height is preserved.
  renderItemLabel: label => label || ' ',
  tabIndex: '0'
};

const stopPropagation = event => {
  event.stopPropagation();
};

class Menu extends _react.Component {
  constructor(props) {
    super(props);

    this.handleBlur = this.handleBlur.bind(this);
    this.handleFocus = this.handleFocus.bind(this);
    this.handleItemClick = this.handleItemClick.bind(this);
    this.handleItemMouseDownCapture = this.handleItemMouseDownCapture.bind(this);
    this.handleItemMouseEnter = this.handleItemMouseEnter.bind(this);
    this.handleItemMouseLeave = this.handleItemMouseLeave.bind(this);
    this.handleKeyDown = this.handleKeyDown.bind(this);
    this.handleKeyPress = this.handleKeyPress.bind(this);
    this.handleRef = this.handleRef.bind(this);
    this.handleSelectedItemRef = this.handleSelectedItemRef.bind(this);

    this.state = {
      focusedIndex: null,
      value: props.value
    };
  }

  componentWillReceiveProps(nextProps) {
    this.setState({
      value: nextProps.value
    });
  }

  componentDidUpdate(prevProps, prevState) {
    if (prevState.focusedIndex !== this.state.focusedIndex && !this.isItemMouseDown) {
      this.scrollFocusedItemIntoView();
    }
  }

  scrollFocusedItemIntoView() {
    const focusedIndex = this.state.focusedIndex;


    if (focusedIndex !== null && this.domNode) {
      const focusedItem = this.domNode.querySelector(`li:nth-of-type(${focusedIndex + 1})`);

      const focusedItemRect = focusedItem.getBoundingClientRect();
      const focusedItemTop = focusedItemRect.top;
      const focusedItemBottom = focusedItemRect.bottom;

      const menuRect = this.domNode.getBoundingClientRect();
      const menuTop = menuRect.top;
      const menuBottom = menuRect.bottom;

      if (focusedItemBottom >= menuBottom) {
        // Scroll the bottom of the item into view.

        const delta = Math.ceil(focusedItemBottom - menuBottom);

        this.domNode.scrollTop = this.domNode.scrollTop + delta;
      } else if (focusedItemTop <= menuTop) {
        // Scroll the top of the item into view.

        const delta = Math.ceil(menuTop - focusedItemTop);

        this.domNode.scrollTop = this.domNode.scrollTop - delta;
      }
    }
  }

  focus(itemIndex) {
    if (typeof itemIndex === 'number') {
      const options = this.props.options;


      this.selectedIndex = itemIndex >= 0 ? itemIndex : options.length + itemIndex;
    }

    if (this.domNode) {
      this.domNode.focus();
    }
  }

  handleBlur() {
    this.setState({
      focusedIndex: null
    });
  }

  handleFocus() {
    this.setState({
      focusedIndex: this.selectedIndex || 0
    });
  }

  handleItemClick(event) {
    const index = parseInt(event.currentTarget.dataset.index, 10);

    this.isItemMouseDown = false;
    this.selectItem(index);
  }

  handleItemMouseDownCapture() {
    this.isItemMouseDown = true;
  }

  handleItemMouseEnter(event) {
    const onItemMouseEnter = this.props.onItemMouseEnter;


    if (onItemMouseEnter) {
      const index = parseInt(event.target.dataset.index, 10);
      const option = this.props.options[index];

      onItemMouseEnter(option.value, event.target, event.relatedTarget);
    }
  }

  handleItemMouseLeave(event) {
    const onItemMouseLeave = this.props.onItemMouseLeave;


    if (onItemMouseLeave) {
      const index = parseInt(event.target.dataset.index, 10);
      const option = this.props.options[index];

      onItemMouseLeave(option.value, event.target, event.relatedTarget);
    }
  }

  handleKeyDown(event) {
    if (event.key === 'ArrowDown' || event.key === 'ArrowUp') {
      // Prevent the page from scrolling.
      event.preventDefault();

      let focusedIndex = this.state.focusedIndex;
      var _props = this.props;
      const options = _props.options,
            onBeforeItemFocusChange = _props.onBeforeItemFocusChange;


      let nextFocusedIndex = focusedIndex;

      if (event.key === 'ArrowDown') {
        nextFocusedIndex += 1;

        if (nextFocusedIndex >= options.length) {
          nextFocusedIndex = 0;
        }
        if (onBeforeItemFocusChange) {
          focusedIndex = onBeforeItemFocusChange(focusedIndex, nextFocusedIndex, event.key);
        } else {
          focusedIndex = nextFocusedIndex;
        }
      } else {
        nextFocusedIndex -= 1;

        if (nextFocusedIndex < 0) {
          nextFocusedIndex = options.length - 1;
        }

        if (onBeforeItemFocusChange) {
          focusedIndex = onBeforeItemFocusChange(focusedIndex, nextFocusedIndex, event.key);
        } else {
          focusedIndex = nextFocusedIndex;
        }
      }
      this.setState({
        focusedIndex
      });
    }
  }

  handleKeyPress(event) {
    if (event.key === 'Enter') {
      this.selectItem(this.state.focusedIndex);
    }
  }

  handleRef(ref) {
    this.domNode = ref;
  }

  handleSelectedItemRef(ref) {
    this.selectedIndex = ref === null ? null : parseInt(ref.dataset.index, 10);
  }

  selectItem(index) {
    var _props2 = this.props;
    const options = _props2.options,
          ignoreDisabledOptions = _props2.ignoreDisabledOptions;


    const option = options[index];

    if (ignoreDisabledOptions || !option.disabled) {
      const value = option.value;

      this.setState({
        value,
        focusedIndex: index
      });

      const onSelect = this.props.onSelect;


      if (onSelect) {
        onSelect(option);
      }
    }
  }

  renderItems() {
    var _state = this.state;
    const focusedIndex = _state.focusedIndex,
          value = _state.value;
    var _props3 = this.props;
    const ignoreDisabledOptions = _props3.ignoreDisabledOptions,
          options = _props3.options,
          renderItemLabel = _props3.renderItemLabel;


    return options.map((option, index) => {
      const optionValue = option.value,
            optionLabel = option.label,
            optionClassName = option.className,
            optionDisabled = option.disabled,
            optionIndent = option.indent,
            optionStartGroup = option.startGroup;


      const className = (0, _classnames2.default)(optionIndent ? _MenuItem2.default[`indent${optionIndent}`] : null, {
        [_MenuItem2.default.common]: true,
        [_MenuItem2.default.startGroup]: !!optionStartGroup,
        [_MenuItem2.default.disabled]: optionDisabled && !ignoreDisabledOptions,
        [_MenuItem2.default.selected]: optionValue === value,
        [_MenuItem2.default.focused]: focusedIndex === index,
        [optionClassName]: true
      });

      const ref = optionValue === value ? this.handleSelectedItemRef : null;

      // TODO: ARIA
      // https://www.w3.org/WAI/GL/wiki/index.php?title=Using_aria-activedescendant&oldid=2629

      return (
        /* eslint-disable jsx-a11y/no-static-element-interactions */
        _react2.default.createElement(
          'li',
          {
            className: className,
            'data-index': index,
            key: index,
            ref: ref,
            role: 'option',
            onClick: this.handleItemClick

            // Prevent flash of incorrectly focused item when clicking on an item in an unfocused
            // menu.
            , onMouseDown: stopPropagation

            // Track if the mouse is down on an item, using the capture phase so that the state
            // of the mouse may be updated before onFocus fires.
            , onMouseDownCapture: this.handleItemMouseDownCapture,

            onMouseEnter: this.handleItemMouseEnter,
            onMouseLeave: this.handleItemMouseLeave
          },
          renderItemLabel(optionLabel)
        )
        /* eslint-enable jsx-a11y/no-static-element-interactions */

      );
    });
  }

  render() {
    const tabIndex = this.props.tabIndex;


    const items = this.renderItems();

    if (items.length === 0) {
      return null;
    }

    // TODO: ARIA
    // https://www.w3.org/WAI/GL/wiki/index.php?title=Using_aria-activedescendant&oldid=2629

    return (
      /* eslint-disable jsx-a11y/no-static-element-interactions */
      _react2.default.createElement(
        'ul',
        {
          className: _Menu2.default.common,
          ref: this.handleRef,
          role: 'listbox',
          tabIndex: tabIndex,
          onBlur: this.handleBlur,
          onFocus: this.handleFocus,
          onKeyDown: this.handleKeyDown,
          onKeyPress: this.handleKeyPress
        },
        items
      )
      /* eslint-enable jsx-a11y/no-static-element-interactions */

    );
  }
}

exports.default = Menu;
Menu.propTypes = propTypes;
Menu.defaultProps = defaultProps;