'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _pathHelpers = require('../helpers/pathHelpers');

var _MultilineInput = require('../../styles/cspace-input/MultilineInput.css');

var _MultilineInput2 = _interopRequireDefault(_MultilineInput);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

const propTypes = {
  asText: _propTypes2.default.bool,
  embedded: _propTypes2.default.bool,
  name: _propTypes2.default.string,
  parentPath: _pathHelpers.pathPropType,
  subpath: _pathHelpers.pathPropType,
  value: _propTypes2.default.string,
  readOnly: _propTypes2.default.bool,
  api: _propTypes2.default.func
};

/**
 * A text input that accepts and is able to display multiple lines of text. This component takes up
 * more screen space when rendered than LineInput.
 */
class MultilineInput extends _react.Component {
  constructor() {
    super();

    this.handleRef = this.handleRef.bind(this);
  }

  componentDidMount() {
    const api = this.props.api;


    if (api) {
      api({
        focus: this.focus.bind(this)
      });
    }
  }

  componentWillUnmount() {
    const api = this.props.api;


    if (api) {
      api(null);
    }
  }

  focus() {
    this.domNode.focus();
  }

  handleRef(ref) {
    this.domNode = ref;
  }

  render() {
    var _props = this.props;

    const asText = _props.asText,
          embedded = _props.embedded,
          value = _props.value,
          readOnly = _props.readOnly,
          parentPath = _props.parentPath,
          subpath = _props.subpath,
          api = _props.api,
          remainingProps = _objectWithoutProperties(_props, ['asText', 'embedded', 'value', 'readOnly', 'parentPath', 'subpath', 'api']);

    const className = embedded ? _MultilineInput2.default.embedded : _MultilineInput2.default.normal;
    const normalizedValue = value === null || typeof value === 'undefined' ? '' : value;

    if (asText) {
      return _react2.default.createElement(
        'div',
        { className: className },
        normalizedValue
      );
    }

    return _react2.default.createElement('textarea', _extends({}, remainingProps, {
      className: className,
      disabled: readOnly,
      readOnly: !remainingProps.onChange,
      ref: this.handleRef,
      value: normalizedValue
    }));
  }
}

exports.default = MultilineInput;
MultilineInput.propTypes = propTypes;