'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

var _Menu = require('./Menu');

var _Menu2 = _interopRequireDefault(_Menu);

var _parseResourceID = require('../helpers/parseResourceID');

var _parseResourceID2 = _interopRequireDefault(_parseResourceID);

var _QuickAdd = require('../../styles/cspace-input/QuickAdd.css');

var _QuickAdd2 = _interopRequireDefault(_QuickAdd);

var _QuickAddItem = require('../../styles/cspace-input/QuickAddItem.css');

var _QuickAddItem2 = _interopRequireDefault(_QuickAddItem);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const propTypes = {
  add: _propTypes2.default.func,
  displayName: _propTypes2.default.string,
  partialTerm: _propTypes2.default.string,
  formatAddPrompt: _propTypes2.default.func,
  formatCloneOptionLabel: _propTypes2.default.func,
  formatCreateNewOptionLabel: _propTypes2.default.func,
  formatDestinationName: _propTypes2.default.func,
  recordTypes: _propTypes2.default.object,
  showCloneOption: _propTypes2.default.bool,
  cloneOptionDisabled: _propTypes2.default.bool,
  to: _propTypes2.default.string,
  onBeforeItemFocusChange: _propTypes2.default.func
};

const defaultProps = {
  formatAddPrompt: (displayName, destinationName) => `Add "${displayName}" to ${destinationName}`,
  formatCloneOptionLabel: () => 'Clone current record',
  formatCreateNewOptionLabel: () => 'Create new record',
  formatDestinationName: (recordTypeConfig, vocabulary) => {
    if (vocabulary) {
      return (0, _get2.default)(recordTypeConfig, ['vocabularies', vocabulary, 'messages', 'collectionName', 'defaultMessage']);
    }

    return (0, _get2.default)(recordTypeConfig, ['messages', 'record', 'collectionName', 'defaultMessage']);
  }
};

class QuickAdd extends _react.Component {
  constructor(props) {
    super(props);

    this.handleMenuRef = this.handleMenuRef.bind(this);
    this.handleMenuSelect = this.handleMenuSelect.bind(this);
  }

  focusMenu(itemIndex) {
    if (this.menu) {
      this.menu.focus(itemIndex);
    }
  }

  handleMenuRef(ref) {
    this.menu = ref;
  }

  handleMenuSelect(item) {
    var _props = this.props;
    const add = _props.add,
          displayName = _props.displayName,
          partialTerm = _props.partialTerm;


    if (add) {
      const clone = item.clone,
            value = item.value;

      var _value$split = value.split('/'),
          _value$split2 = _slicedToArray(_value$split, 2);

      const recordType = _value$split2[0],
            vocabulary = _value$split2[1];


      add(recordType, vocabulary, displayName, partialTerm, clone);
    }
  }

  render() {
    var _props2 = this.props;
    const displayName = _props2.displayName,
          formatAddPrompt = _props2.formatAddPrompt,
          formatCloneOptionLabel = _props2.formatCloneOptionLabel,
          formatCreateNewOptionLabel = _props2.formatCreateNewOptionLabel,
          formatDestinationName = _props2.formatDestinationName,
          recordTypes = _props2.recordTypes,
          showCloneOption = _props2.showCloneOption,
          cloneOptionDisabled = _props2.cloneOptionDisabled,
          destinationID = _props2.to,
          onBeforeItemFocusChange = _props2.onBeforeItemFocusChange;


    const destinations = (0, _parseResourceID2.default)(destinationID);

    const options = destinations.map(destination => {
      const recordType = destination.recordType,
            vocabulary = destination.vocabulary;


      const recordTypeConfig = (0, _get2.default)(recordTypes, recordType);

      if (!recordTypeConfig) {
        return null;
      }

      if (vocabulary) {
        const vocabularyConfig = (0, _get2.default)(recordTypeConfig, ['vocabularies', vocabulary]);

        if (!vocabularyConfig) {
          return null;
        }
      }

      return {
        label: formatDestinationName(recordTypeConfig, vocabulary),
        value: `${recordType}/${vocabulary}`,
        className: _QuickAddItem2.default.add
      };
    }).filter(option => !!option);

    if (options.length === 0) {
      return null;
    }

    // If showCloneOption is true and there is only one destination, the options should be to
    // create new or clone into the source.

    let singleDestinationName;

    if (options.length === 1 && showCloneOption) {
      singleDestinationName = options[0].label;

      options[0].label = formatCreateNewOptionLabel();

      options.push(Object.assign({}, options[0], {
        clone: true,
        label: formatCloneOptionLabel(),
        className: _QuickAddItem2.default.clone,
        disabled: cloneOptionDisabled
      }));
    }

    return _react2.default.createElement(
      'div',
      { className: _QuickAdd2.default.common },
      _react2.default.createElement(
        'div',
        null,
        formatAddPrompt(displayName, singleDestinationName)
      ),
      _react2.default.createElement(_Menu2.default, {
        options: options,
        tabIndex: '-1',
        ref: this.handleMenuRef,
        onBeforeItemFocusChange: onBeforeItemFocusChange,
        onSelect: this.handleMenuSelect
      })
    );
  }
}

exports.default = QuickAdd;
QuickAdd.propTypes = propTypes;
QuickAdd.defaultProps = defaultProps;