'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _Button = require('./Button');

var _Button2 = _interopRequireDefault(_Button);

var _LineInput = require('./LineInput');

var _LineInput2 = _interopRequireDefault(_LineInput);

var _RecordTypeInput = require('./RecordTypeInput');

var _RecordTypeInput2 = _interopRequireDefault(_RecordTypeInput);

var _VocabularyInput = require('./VocabularyInput');

var _VocabularyInput2 = _interopRequireDefault(_VocabularyInput);

var _changeable = require('../enhancers/changeable');

var _changeable2 = _interopRequireDefault(_changeable);

var _committable = require('../enhancers/committable');

var _committable2 = _interopRequireDefault(_committable);

var _QuickSearchInput = require('../../styles/cspace-input/QuickSearchInput.css');

var _QuickSearchInput2 = _interopRequireDefault(_QuickSearchInput);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const LineInput = (0, _committable2.default)((0, _changeable2.default)(_LineInput2.default));

const propTypes = {
  formatRecordTypeLabel: _propTypes2.default.func,
  formatVocabularyLabel: _propTypes2.default.func,
  keywordValue: _propTypes2.default.string,
  placeholder: _propTypes2.default.string,
  searchButtonLabel: _propTypes2.default.string,
  recordTypes: _propTypes2.default.object,
  recordTypeValue: _propTypes2.default.string,
  vocabularyValue: _propTypes2.default.string,
  onKeywordCommit: _propTypes2.default.func,
  onRecordTypeCommit: _propTypes2.default.func,
  onVocabularyCommit: _propTypes2.default.func,
  search: _propTypes2.default.func
};

class QuickSearchInput extends _react.Component {
  constructor() {
    super();

    this.handleButtonClick = this.handleButtonClick.bind(this);
    this.handleKeyPress = this.handleKeyPress.bind(this);
    this.handleKeywordInputCommit = this.handleKeywordInputCommit.bind(this);
    this.handleRecordTypeDropdownCommit = this.handleRecordTypeDropdownCommit.bind(this);
    this.handleRecordTypeDropdownUpdate = this.handleRecordTypeDropdownUpdate.bind(this);
    this.handleVocabularyDropdownCommit = this.handleVocabularyDropdownCommit.bind(this);
    this.handleVocabularyDropdownUpdate = this.handleVocabularyDropdownUpdate.bind(this);
  }

  handleButtonClick() {
    const search = this.props.search;


    if (search) {
      search();
    }
  }

  handleKeywordInputCommit(path, value) {
    const onKeywordCommit = this.props.onKeywordCommit;


    if (onKeywordCommit) {
      onKeywordCommit(value);
    }
  }

  handleKeyPress(event) {
    if (event.key === 'Enter') {
      const search = this.props.search;


      if (search) {
        search();
      }
    }
  }

  handleRecordTypeDropdownCommit(path, value) {
    const onRecordTypeCommit = this.props.onRecordTypeCommit;


    if (onRecordTypeCommit) {
      onRecordTypeCommit(value);
    }
  }

  handleRecordTypeDropdownUpdate({ value }) {
    if (value !== this.props.recordTypeValue) {
      // A default was selected.

      const onRecordTypeCommit = this.props.onRecordTypeCommit;


      if (onRecordTypeCommit) {
        onRecordTypeCommit(value);
      }
    }
  }

  handleVocabularyDropdownCommit(path, value) {
    const onVocabularyCommit = this.props.onVocabularyCommit;


    if (onVocabularyCommit) {
      onVocabularyCommit(value);
    }
  }

  handleVocabularyDropdownUpdate({ value }) {
    if (value !== this.props.vocabularyValue) {
      // A default was selected.

      const onVocabularyCommit = this.props.onVocabularyCommit;


      if (onVocabularyCommit) {
        onVocabularyCommit(value);
      }
    }
  }

  renderRecordTypeDropdown() {
    var _props = this.props;
    const formatRecordTypeLabel = _props.formatRecordTypeLabel,
          recordTypes = _props.recordTypes,
          recordTypeValue = _props.recordTypeValue;


    return _react2.default.createElement(_RecordTypeInput2.default, {
      embedded: true,
      formatRecordTypeLabel: formatRecordTypeLabel,
      recordTypes: recordTypes,
      value: recordTypeValue,
      onCommit: this.handleRecordTypeDropdownCommit,
      onKeyPress: this.handleKeyPress,
      onMount: this.handleRecordTypeDropdownUpdate,
      onUpdate: this.handleRecordTypeDropdownUpdate
    });
  }

  renderVocabularyDropdown() {
    var _props2 = this.props;
    const formatVocabularyLabel = _props2.formatVocabularyLabel,
          recordTypes = _props2.recordTypes,
          recordTypeValue = _props2.recordTypeValue,
          vocabularyValue = _props2.vocabularyValue;


    return _react2.default.createElement(_VocabularyInput2.default, {
      embedded: true,
      formatVocabularyLabel: formatVocabularyLabel,
      recordTypes: recordTypes,
      recordType: recordTypeValue,
      value: vocabularyValue,
      onCommit: this.handleVocabularyDropdownCommit,
      onKeyPress: this.handleKeyPress,
      onMount: this.handleVocabularyDropdownUpdate,
      onUpdate: this.handleVocabularyDropdownUpdate
    });
  }

  renderKeywordInput() {
    var _props3 = this.props;
    const keywordValue = _props3.keywordValue,
          placeholder = _props3.placeholder;


    return _react2.default.createElement(LineInput, {
      embedded: true,
      placeholder: placeholder,
      value: keywordValue,
      onCommit: this.handleKeywordInputCommit,
      onKeyPress: this.handleKeyPress
    });
  }

  render() {
    const searchButtonLabel = this.props.searchButtonLabel;


    return _react2.default.createElement(
      'div',
      { className: _QuickSearchInput2.default.normal },
      _react2.default.createElement(
        'div',
        null,
        this.renderRecordTypeDropdown(),
        this.renderVocabularyDropdown(),
        this.renderKeywordInput()
      ),
      _react2.default.createElement(_Button2.default, {
        'aria-label': searchButtonLabel,
        onClick: this.handleButtonClick
      })
    );
  }
}

exports.default = QuickSearchInput;
QuickSearchInput.propTypes = propTypes;