'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = RecordTypeInput;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _SubstringFilteringDropdownMenuInput = require('./SubstringFilteringDropdownMenuInput');

var _SubstringFilteringDropdownMenuInput2 = _interopRequireDefault(_SubstringFilteringDropdownMenuInput);

var _DropdownMenuInput = require('./DropdownMenuInput');

var _DropdownMenuInput2 = _interopRequireDefault(_DropdownMenuInput);

var _withLabeledOptions = require('../enhancers/withLabeledOptions');

var _withLabeledOptions2 = _interopRequireDefault(_withLabeledOptions);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

const SubstringFilteringDropdownMenuInput = (0, _withLabeledOptions2.default)(_SubstringFilteringDropdownMenuInput2.default);

const DropdownMenuInput = (0, _withLabeledOptions2.default)(_DropdownMenuInput2.default);

const propTypes = {
  formatRecordTypeLabel: _propTypes2.default.func,
  indentItems: _propTypes2.default.bool,
  recordTypes: _propTypes2.default.object,
  rootType: _propTypes2.default.string,
  serviceTypes: _propTypes2.default.arrayOf(_propTypes2.default.string),
  value: _propTypes2.default.string,
  filtering: _propTypes2.default.bool
};

const defaultProps = {
  filtering: true,
  formatRecordTypeLabel: (name, config) => {
    const messages = config.messages;

    if (messages) {
      return messages.record.collectionName.defaultMessage;
    }

    return name;
  },
  indentItems: true,
  recordTypes: {},
  rootType: 'all',
  serviceTypes: ['object', 'procedure', 'authority']
};

function RecordTypeInput(props) {
  const formatRecordTypeLabel = props.formatRecordTypeLabel,
        indentItems = props.indentItems,
        recordTypes = props.recordTypes,
        rootType = props.rootType,
        serviceTypes = props.serviceTypes,
        filtering = props.filtering,
        recordTypeValue = props.value,
        remainingProps = _objectWithoutProperties(props, ['formatRecordTypeLabel', 'indentItems', 'recordTypes', 'rootType', 'serviceTypes', 'filtering', 'value']);

  const options = [];

  if (recordTypes[rootType]) {
    options.push({
      value: rootType,
      label: formatRecordTypeLabel(rootType, recordTypes[rootType])
    });
  }

  serviceTypes.forEach(serviceType => {
    const recordTypeNames = Object.keys(recordTypes).filter(recordTypeName => {
      const recordTypeConfig = recordTypes[recordTypeName];

      return recordTypeConfig.serviceConfig.serviceType === serviceType && !recordTypeConfig.disabled;
    }).sort((nameA, nameB) => {
      const configA = recordTypes[nameA];
      const configB = recordTypes[nameB];

      // Primary sort by sortOrder

      let sortOrderA = configA.sortOrder;
      let sortOrderB = configB.sortOrder;

      if (typeof sortOrderA !== 'number') {
        sortOrderA = Number.MAX_VALUE;
      }

      if (typeof sortOrderB !== 'number') {
        sortOrderB = Number.MAX_VALUE;
      }

      if (sortOrderA !== sortOrderB) {
        return sortOrderA > sortOrderB ? 1 : -1;
      }

      // Secondary sort by label

      const labelA = formatRecordTypeLabel(nameA, configA);
      const labelB = formatRecordTypeLabel(nameB, configB);

      // FIXME: This should be locale aware
      return labelA.localeCompare(labelB);
    });

    let indent = 0;

    if (recordTypeNames.length > 1 && recordTypes[serviceType]) {
      // If there is more than one record type within this service type, and the service type is
      // itself a record type, add the service type as a parent option.

      if (indentItems) {
        indent += 1;
      }

      options.push({
        indent,
        value: serviceType,
        label: formatRecordTypeLabel(serviceType, recordTypes[serviceType])
      });
    }

    if (indentItems) {
      indent += 1;
    }

    recordTypeNames.forEach(recordTypeName => {
      options.push({
        indent,
        value: recordTypeName,
        label: formatRecordTypeLabel(recordTypeName, recordTypes[recordTypeName])
      });
    });
  });

  let value = recordTypeValue;

  if (!value) {
    for (let i = 0; i < options.length; i += 1) {
      const recordTypeName = options[i].value;
      const recordType = recordTypes[recordTypeName];

      if (recordType.defaultForSearch) {
        value = recordTypeName;
        break;
      }
    }

    if (!value && options.length > 0) {
      value = options[0].value;
    }
  }

  const Input = filtering ? SubstringFilteringDropdownMenuInput : DropdownMenuInput;

  return _react2.default.createElement(Input, _extends({
    blankable: false,
    options: options,
    value: value
  }, remainingProps));
}

RecordTypeInput.propTypes = propTypes;
RecordTypeInput.defaultProps = defaultProps;