'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

var _MiniButton = require('./MiniButton');

var _MiniButton2 = _interopRequireDefault(_MiniButton);

var _normalizeLabel = require('../helpers/normalizeLabel');

var _normalizeLabel2 = _interopRequireDefault(_normalizeLabel);

var _pathHelpers = require('../helpers/pathHelpers');

var _MiniButtonContainer = require('../../styles/cspace-input/MiniButtonContainer.css');

var _MiniButtonContainer2 = _interopRequireDefault(_MiniButtonContainer);

var _MoveToTopButton = require('../../styles/cspace-input/MoveToTopButton.css');

var _MoveToTopButton2 = _interopRequireDefault(_MoveToTopButton);

var _RepeatingInput = require('../../styles/cspace-input/RepeatingInput.css');

var _RepeatingInput2 = _interopRequireDefault(_RepeatingInput);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const normalizeValue = value => {
  const defaultValue = [undefined];

  if (!value) {
    return defaultValue;
  }

  let normalized;

  if (Array.isArray(value)) {
    normalized = value;
  } else if (_immutable2.default.List.isList(value)) {
    normalized = value.toArray();
  } else {
    normalized = [value];
  }

  if (normalized.length === 0) {
    return defaultValue;
  }

  return normalized;
};

const renderHeader = label => label;

const propTypes = {
  children: _propTypes2.default.node,
  name: _propTypes2.default.string,
  parentPath: _pathHelpers.pathPropType, // eslint-disable-line react/no-unused-prop-types
  subpath: _pathHelpers.pathPropType, // eslint-disable-line react/no-unused-prop-types
  value: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.object, _propTypes2.default.instanceOf(_immutable2.default.List), _propTypes2.default.arrayOf(_propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.object]))]),
  readOnly: _propTypes2.default.bool,
  asText: _propTypes2.default.bool,
  reorderable: _propTypes2.default.bool,
  renderOrderIndicator: _propTypes2.default.func,
  onAddInstance: _propTypes2.default.func,
  onCommit: _propTypes2.default.func,
  onMoveInstance: _propTypes2.default.func,
  onRemoveInstance: _propTypes2.default.func
};

const defaultProps = {
  reorderable: true
};

class RepeatingInput extends _react.Component {
  constructor(props) {
    super(props);

    this.handleInstanceCommit = this.handleInstanceCommit.bind(this);
    this.handleAddButtonClick = this.handleAddButtonClick.bind(this);
    this.handleMoveToTopButtonClick = this.handleMoveToTopButtonClick.bind(this);
    this.handleMoveToTopButtonKeyDown = this.handleMoveToTopButtonKeyDown.bind(this);
    this.handleInstancesContainerRef = this.handleInstancesContainerRef.bind(this);
    this.handleRemoveButtonClick = this.handleRemoveButtonClick.bind(this);
  }

  getLabel() {
    const children = this.props.children;


    const template = _react2.default.Children.only(children);

    const label = template.props.label;


    return (0, _normalizeLabel2.default)(label);
  }

  focus(position) {
    if (this.instancesContainerNode) {
      // FIXME: This is a bit of a hack. It should use refs to track the move to top button nodes.

      const instance = this.instancesContainerNode.querySelector(`div[data-instancename="${position}"]`);
      const moveToTopButtonInstance = instance.querySelector('.cspace-input-MoveToTopButton--common');

      if (moveToTopButtonInstance) {
        moveToTopButtonInstance.focus();
      }
    }
  }

  handleAddButtonClick() {
    const onAddInstance = this.props.onAddInstance;


    if (onAddInstance) {
      onAddInstance((0, _pathHelpers.getPath)(this.props));
    }
  }

  handleInstanceCommit(instancePath, value) {
    const onCommit = this.props.onCommit;


    if (onCommit) {
      onCommit(instancePath, value);
    }
  }

  handleInstancesContainerRef(ref) {
    this.instancesContainerNode = ref;
  }

  handleMoveToTopButtonClick(event) {
    const onMoveInstance = this.props.onMoveInstance;


    if (onMoveInstance) {
      const instanceName = event.target.dataset.instancename;
      const newPosition = 0;

      onMoveInstance([...(0, _pathHelpers.getPath)(this.props), instanceName], newPosition);
    }
  }

  handleMoveToTopButtonKeyDown(event) {
    var _props = this.props;
    const value = _props.value,
          onAddInstance = _props.onAddInstance,
          onMoveInstance = _props.onMoveInstance;


    if (event.shiftKey) {
      const key = event.key,
            currentTarget = event.currentTarget;


      if (key === 'ArrowDown' || key === 'ArrowUp') {
        if (onMoveInstance) {
          const instanceName = currentTarget.dataset.instancename;
          const position = parseInt(instanceName, 10);
          const normalizedValue = normalizeValue(value);

          let newPosition;

          if (key === 'ArrowDown') {
            newPosition = position + 1;

            if (newPosition > normalizedValue.length - 1) {
              newPosition = 0;
            }
          } else {
            newPosition = position - 1;

            if (newPosition < 0) {
              newPosition = normalizedValue.length - 1;
            }
          }

          onMoveInstance([...(0, _pathHelpers.getPath)(this.props), instanceName], newPosition);

          this.focus(newPosition);
        }
      } else if (key === '+') {
        if (onAddInstance) {
          const instanceName = currentTarget.dataset.instancename;
          const position = parseInt(instanceName, 10);

          onAddInstance((0, _pathHelpers.getPath)(this.props), position + 1);
        }
      }
    }
  }

  handleRemoveButtonClick(event) {
    const onRemoveInstance = this.props.onRemoveInstance;


    if (onRemoveInstance) {
      const instanceName = event.target.dataset.instancename;

      onRemoveInstance([...(0, _pathHelpers.getPath)(this.props), instanceName]);
    }
  }

  renderEmbeddedHeader(label) {
    const readOnly = this.props.readOnly;


    let removeButtonCol;

    if (!readOnly) {
      removeButtonCol = _react2.default.createElement('div', null);
    }

    return _react2.default.createElement(
      'header',
      null,
      _react2.default.createElement('div', null),
      _react2.default.createElement(
        'div',
        null,
        label
      ),
      removeButtonCol
    );
  }

  renderInstances() {
    var _props2 = this.props;
    const asText = _props2.asText,
          children = _props2.children,
          value = _props2.value,
          readOnly = _props2.readOnly,
          reorderable = _props2.reorderable,
          renderOrderIndicator = _props2.renderOrderIndicator;


    const template = _react2.default.Children.only(children);
    const normalizedValue = normalizeValue(value);

    return normalizedValue.map((instanceValue, index, list) => {
      const instanceName = `${index}`;

      const overrideProps = {
        asText,
        readOnly,
        embedded: true,
        label: undefined,
        name: instanceName,
        parentPath: (0, _pathHelpers.getPath)(this.props),
        value: instanceValue,
        // The template is expected to accept an onCommit prop.
        onCommit: this.handleInstanceCommit
      };

      const inputInstance = _react2.default.cloneElement(template, overrideProps);

      let orderIndicator = null;

      const hideOrderNumber = readOnly && normalizedValue.length === 1 && !normalizedValue[0];
      const orderNumber = hideOrderNumber ? null : index + 1;

      if (renderOrderIndicator) {
        orderIndicator = renderOrderIndicator(orderNumber);
      } else {
        const className = readOnly ? _MiniButtonContainer2.default.readOnly : _MiniButtonContainer2.default.normal;

        if (asText) {
          orderIndicator = _react2.default.createElement(
            'div',
            null,
            orderNumber
          );
        } else {
          orderIndicator = _react2.default.createElement(
            'div',
            { className: className },
            _react2.default.createElement(
              _MiniButton2.default,
              {
                className: _MoveToTopButton2.default.common,
                'data-instancename': instanceName,
                disabled: !reorderable,
                name: 'moveToTop',
                readOnly: readOnly,
                onClick: this.handleMoveToTopButtonClick,
                onKeyDown: this.handleMoveToTopButtonKeyDown
              },
              orderNumber
            )
          );
        }
      }

      let removeButton;

      if (!readOnly && !asText) {
        const className = _MiniButtonContainer2.default.normal;

        removeButton = _react2.default.createElement(
          'div',
          { className: className },
          _react2.default.createElement(
            _MiniButton2.default,
            {
              'data-instancename': instanceName,
              disabled: list.length < 2,
              name: 'remove',
              onClick: this.handleRemoveButtonClick
            },
            '\u2212'
          )
        );
      }

      return _react2.default.createElement(
        'div',
        {
          'data-instancename': instanceName,
          key: index
        },
        orderIndicator,
        _react2.default.createElement(
          'div',
          null,
          inputInstance
        ),
        removeButton
      );
    });
  }

  render() {
    var _props3 = this.props;
    const asText = _props3.asText,
          name = _props3.name,
          readOnly = _props3.readOnly;


    const label = this.getLabel();
    const isLabelEmbedded = (0, _get2.default)(label, ['props', 'embedded']);

    const instances = this.renderInstances();

    if (asText) {
      return _react2.default.createElement(
        'div',
        null,
        instances
      );
    }

    let footer;

    if (!readOnly) {
      const className = _MiniButtonContainer2.default.normal;

      footer = _react2.default.createElement(
        'footer',
        null,
        _react2.default.createElement(
          'div',
          { className: className },
          _react2.default.createElement(
            _MiniButton2.default,
            {
              name: 'add',
              onClick: this.handleAddButtonClick
            },
            '+'
          )
        )
      );
    }

    const className = readOnly ? _RepeatingInput2.default.readOnly : _RepeatingInput2.default.normal;

    return _react2.default.createElement(
      'fieldset',
      {
        className: className,
        name: name
      },
      isLabelEmbedded ? null : renderHeader(label),
      _react2.default.createElement(
        'div',
        { ref: this.handleInstancesContainerRef },
        isLabelEmbedded ? this.renderEmbeddedHeader(label) : null,
        instances
      ),
      footer
    );
  }
}

exports.default = RepeatingInput;
RepeatingInput.propTypes = propTypes;
RepeatingInput.defaultProps = defaultProps;