'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactQuill = require('react-quill');

var _reactQuill2 = _interopRequireDefault(_reactQuill);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _pathHelpers = require('../helpers/pathHelpers');

var _RichTextInput = require('../../styles/cspace-input/RichTextInput.css');

var _RichTextInput2 = _interopRequireDefault(_RichTextInput);

require('!style-loader!css-loader!../../styles/quill/cspace.css');

require('!style-loader!css-loader!react-quill/dist/quill.core.css');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-enable import/imports-first, import/no-unresolved */

// Change bold rendering to <b> instead of <strong>, since it is more semantically neutral.

/* eslint-disable import/imports-first, import/no-unresolved */
/* global window */

const Bold = _reactQuill.Quill.import('formats/bold');
Bold.tagName = 'B';
_reactQuill.Quill.register(Bold, true);

// Change italic rendering to <i> instead of <em>, since it is more semantically neutral.

const Italic = _reactQuill.Quill.import('formats/italic');
Italic.tagName = 'I';
_reactQuill.Quill.register(Italic, true);

const normalizeValue = value => {
  let normalizedValue = value;

  if (normalizedValue) {
    // If there is only a single paragraph, remove the <p></p> wrapper.

    if (normalizedValue.indexOf('<p>', 1) < 0) {
      normalizedValue = normalizedValue.replace(/^<p>|<\/p>$/g, '');
    }

    // If content is '<br>', set it to empty. Quill uses this as a placeholder.

    if (normalizedValue === '<br>') {
      normalizedValue = '';
    }
  }

  return normalizedValue;
};

const preventButtonMouseDown = event => {
  if (event.target.nodeName === 'BUTTON') {
    // On mouseDown on a toolbar button, Firefox fires a blur event with null relatedTarget. This
    // would be fine, if relatedTarget were correctly set to the toolbar button. But since it's
    // null, the toolbar disappears. Prevent this.

    event.preventDefault();
  }
};

const propTypes = {
  embedded: _propTypes2.default.bool,
  multiline: _propTypes2.default.bool,
  /* eslint-disable react/no-unused-prop-types */
  name: _propTypes2.default.string,
  parentPath: _pathHelpers.pathPropType,
  subpath: _pathHelpers.pathPropType,
  /* eslint-enable react/no-unused-prop-types */
  value: _propTypes2.default.string,
  readOnly: _propTypes2.default.bool,
  onCommit: _propTypes2.default.func
};

class RichTextInput extends _react.Component {
  constructor(props) {
    super(props);

    // Set the allowed formats.
    // TODO: Make this a prop.
    // TODO: Add paragraph formatting (e.g. lists) when multiline is true?

    this.formats = ['bold', 'italic', 'underline', 'script'];

    // Configure the toolbar buttons.
    // TODO: Make this a prop.

    this.modules = {
      keyboard: {
        bindings: {
          // Only allow newlines in content if multiline prop is true.

          enter: {
            key: 13,
            handler: () => {
              this.commit();

              return !!this.props.multiline;
            }
          },

          // Don't allow tabs in the content, just do the browser default of focusing the next
          // field.

          tab: null
        }
      },
      toolbar: ['bold', 'italic', 'underline', { script: 'sub' }, { script: 'super' }, 'clean']
    };

    // TODO: Make a custom HTML toolbar for accessibility:
    // https://github.com/zenoamaro/react-quill#html-toolbar

    this.handleBlur = this.handleBlur.bind(this);
    this.handleFocus = this.handleFocus.bind(this);
    this.handleChange = this.handleChange.bind(this);
    this.handleRef = this.handleRef.bind(this);

    this.state = {
      value: this.props.value
    };
  }

  componentWillReceiveProps(nextProps) {
    this.setState({
      value: nextProps.value
    });
  }

  shouldComponentUpdate(nextProps, nextState) {
    var _props = this.props;
    const embedded = _props.embedded,
          multiline = _props.multiline,
          readOnly = _props.readOnly,
          value = _props.value;


    return nextProps.embedded !== embedded || nextProps.multiline !== multiline || nextProps.readOnly !== readOnly || nextProps.value !== value || nextState.focused !== this.state.focused;
  }

  commit() {
    var _props2 = this.props;
    const onCommit = _props2.onCommit,
          prevValue = _props2.value;


    if (onCommit) {
      const value = normalizeValue(this.state.value);

      if (value !== prevValue) {
        onCommit((0, _pathHelpers.getPath)(this.props), value);
      }
    }
  }

  handleBlur(event) {
    if (!this.domNode.contains(event.relatedTarget)) {
      window.setTimeout(() => {
        this.setState({
          focused: false
        });

        this.commit();
      }, 0);
    }
  }

  handleChange(value) {
    this.setState({
      value
    });
  }

  handleFocus(event) {
    if (this.domNode && !this.domNode.contains(event.relatedTarget)) {
      this.setState({
        focused: true
      });
    }
  }

  handleRef(ref) {
    this.domNode = ref;
  }

  render() {
    var _props3 = this.props;
    const embedded = _props3.embedded,
          multiline = _props3.multiline,
          readOnly = _props3.readOnly;
    var _state = this.state;
    const focused = _state.focused,
          value = _state.value;


    const normalizedValue = value || '';

    const className = (0, _classnames2.default)({
      [_RichTextInput2.default.embedded]: embedded,
      [_RichTextInput2.default.normal]: !embedded,
      [_RichTextInput2.default.enabled]: !readOnly,
      [_RichTextInput2.default.multiline]: multiline,
      [_RichTextInput2.default.focus]: focused
    });

    // The react-quill onBlur event is unreliable, so the wrapper div here is used to do our own
    // implementation.
    // https://github.com/zenoamaro/react-quill/issues/286
    // https://github.com/zenoamaro/react-quill/issues/276

    return _react2.default.createElement(
      'div',
      {
        ref: this.handleRef,
        onBlur: this.handleBlur,
        onFocus: this.handleFocus,
        onMouseDown: preventButtonMouseDown
      },
      _react2.default.createElement(_reactQuill2.default, {
        className: className,
        formats: this.formats,
        modules: this.modules,
        readOnly: readOnly,
        theme: 'snow',
        value: normalizedValue,
        onChange: this.handleChange
      })
    );
  }
}

exports.default = RichTextInput;
RichTextInput.propTypes = propTypes;