'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _Row = require('cspace-layout/lib/components/Row');

var _Row2 = _interopRequireDefault(_Row);

var _set = require('lodash/set');

var _set2 = _interopRequireDefault(_set);

var _merge = require('lodash/merge');

var _merge2 = _interopRequireDefault(_merge);

var _CustomCompoundInput = require('./CustomCompoundInput');

var _CustomCompoundInput2 = _interopRequireDefault(_CustomCompoundInput);

var _DropdownInput = require('./DropdownInput');

var _DropdownInput2 = _interopRequireDefault(_DropdownInput);

var _Label = require('./Label');

var _Label2 = _interopRequireDefault(_Label);

var _TextInput = require('./TextInput');

var _TextInput2 = _interopRequireDefault(_TextInput);

var _SubstringFilteringDropdownMenuInput = require('./SubstringFilteringDropdownMenuInput');

var _SubstringFilteringDropdownMenuInput2 = _interopRequireDefault(_SubstringFilteringDropdownMenuInput);

var _TermPickerInput = require('./TermPickerInput');

var _TermPickerInput2 = _interopRequireDefault(_TermPickerInput);

var _changeable = require('../enhancers/changeable');

var _changeable2 = _interopRequireDefault(_changeable);

var _committable = require('../enhancers/committable');

var _committable2 = _interopRequireDefault(_committable);

var _withLabeledOptions = require('../enhancers/withLabeledOptions');

var _withLabeledOptions2 = _interopRequireDefault(_withLabeledOptions);

var _labelable = require('../enhancers/labelable');

var _labelable2 = _interopRequireDefault(_labelable);

var _pathHelpers = require('../helpers/pathHelpers');

var _dateHelpers = require('../helpers/dateHelpers');

var _StructuredDateInput = require('../../styles/cspace-input/StructuredDateInput.css');

var _StructuredDateInput2 = _interopRequireDefault(_StructuredDateInput);

var _Message = require('../../styles/cspace-input/Message.css');

var _Message2 = _interopRequireDefault(_Message);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

const DropdownInput = (0, _committable2.default)((0, _changeable2.default)(_DropdownInput2.default));
const TextInput = (0, _committable2.default)((0, _changeable2.default)(_TextInput2.default));
const LabelableTextInput = (0, _labelable2.default)(TextInput);

const SubstringFilteringDropdownMenuInput = (0, _withLabeledOptions2.default)(_SubstringFilteringDropdownMenuInput2.default);

const primaryFieldName = 'dateDisplayDate';

const getStructDateFieldValue = (structDateValue, fieldName) => {
  if (!structDateValue) {
    return undefined;
  }

  if (_immutable2.default.Map.isMap(structDateValue)) {
    return structDateValue.get(fieldName);
  }

  return structDateValue[fieldName];
};

const getPrimaryValue = structDateValue => getStructDateFieldValue(structDateValue, primaryFieldName);

const fieldLabels = {
  earliestSingle: 'Earliest/Single',
  latest: 'Latest',
  datePeriod: 'Period',
  dateAssociation: 'Association',
  dateNote: 'Note',
  dateYear: 'Year',
  dateMonth: 'Month',
  dateDay: 'Day',
  dateEra: 'Era',
  dateCertainty: 'Certainty',
  dateQualifier: 'Qualifier',
  dateQualifierValue: 'Value',
  dateQualifierUnit: 'Unit'
};

const propTypes = {
  defaultValue: _propTypes2.default.oneOfType([_propTypes2.default.object, // eslint-disable-line react/forbid-prop-types
  _propTypes2.default.instanceOf(_immutable2.default.Map)]),
  embedded: _propTypes2.default.bool,
  name: _propTypes2.default.string,
  optionLists: _propTypes2.default.object,
  parentPath: _pathHelpers.pathPropType,
  subpath: _pathHelpers.pathPropType,
  terms: _propTypes2.default.object,
  value: _propTypes2.default.oneOfType([_propTypes2.default.object, // eslint-disable-line react/forbid-prop-types
  _propTypes2.default.instanceOf(_immutable2.default.Map)]),
  readOnly: _propTypes2.default.bool,
  formatFieldLabel: _propTypes2.default.func,
  formatOptionLabel: _propTypes2.default.func,
  formatParseFailedMessage: _propTypes2.default.func,
  parseDisplayDate: _propTypes2.default.func,
  onCommit: _propTypes2.default.func,
  onMount: _propTypes2.default.func
};

const defaultProps = {
  defaultValue: {},
  formatFieldLabel: name => fieldLabels[name],
  formatParseFailedMessage: () => 'Unrecognized display date format. Try a different format, or enter values in the fields below.',
  optionLists: {},
  terms: {}
};

class StructuredDateInput extends _react.Component {
  constructor(props) {
    super(props);

    this.handleDropdownInputClose = this.handleDropdownInputClose.bind(this);
    this.handleDropdownInputOpen = this.handleDropdownInputOpen.bind(this);
    this.handleInputCommit = this.handleInputCommit.bind(this);
    this.handlePrimaryInputChange = this.handlePrimaryInputChange.bind(this);
    this.handlePrimaryInputCommit = this.handlePrimaryInputCommit.bind(this);

    const value = props.value || props.defaultValue;

    this.state = {
      value,
      open: false,
      primaryValue: getPrimaryValue(value)
    };
  }

  componentDidMount() {
    const onMount = this.props.onMount;


    if (onMount) {
      onMount();
    }
  }

  componentWillReceiveProps(nextProps) {
    const value = nextProps.value || nextProps.defaultValue;

    this.setState({
      value,
      primaryValue: getPrimaryValue(value)
    });
  }

  getValue(name) {
    const value = this.state.value;


    return getStructDateFieldValue(value, name);
  }

  setValue(path, value) {
    const structDateValue = this.state.value;


    let newStructDateValue;

    if (_immutable2.default.Map.isMap(structDateValue)) {
      if (path.length === 0) {
        newStructDateValue = value;
      } else {
        newStructDateValue = structDateValue.setIn(path, value);
      }

      const newStructDateValueAsObject = newStructDateValue.toJS();
      const dateEarliestScalarValue = (0, _dateHelpers.computeEarliestScalarDate)(newStructDateValueAsObject);
      const dateLatestScalarValue = (0, _dateHelpers.computeLatestScalarDate)(newStructDateValueAsObject);

      newStructDateValue = newStructDateValue.set('dateEarliestScalarValue', dateEarliestScalarValue);
      newStructDateValue = newStructDateValue.set('dateLatestScalarValue', dateLatestScalarValue);
      newStructDateValue = newStructDateValue.set('scalarValuesComputed', true);
    } else {
      if (path.length === 0) {
        newStructDateValue = value;
      } else {
        newStructDateValue = (0, _merge2.default)({}, structDateValue);
      }

      (0, _set2.default)(newStructDateValue, path, value);

      const dateEarliestScalarValue = (0, _dateHelpers.computeEarliestScalarDate)(newStructDateValue);
      const dateLatestScalarValue = (0, _dateHelpers.computeLatestScalarDate)(newStructDateValue);

      newStructDateValue.dateEarliestScalarValue = dateEarliestScalarValue;
      newStructDateValue.dateLatestScalarValue = dateLatestScalarValue;
      newStructDateValue.scalarValuesComputed = true;
    }

    return newStructDateValue;
  }

  handleDropdownInputClose() {
    this.setState({
      open: false
    });
  }

  handleDropdownInputOpen() {
    this.setState({
      open: true
    });
  }

  handleInputCommit(path, value) {
    const nextStructDateValue = this.setValue(path, value);

    var _props = this.props;
    const onCommit = _props.onCommit,
          parseDisplayDate = _props.parseDisplayDate;


    if (onCommit) {
      onCommit((0, _pathHelpers.getPath)(this.props), nextStructDateValue);
    }

    if (parseDisplayDate) {
      const prevDisplayDate = getStructDateFieldValue(this.state.value, 'dateDisplayDate');
      const nextDisplayDate = getStructDateFieldValue(nextStructDateValue, 'dateDisplayDate');

      if (nextDisplayDate !== prevDisplayDate) {
        this.setState({
          parseFailed: false
        });

        parseDisplayDate(nextDisplayDate).then(result => {
          if (!result || result.isError) {
            this.setState({
              parseFailed: true
            });
          } else {
            this.handleInputCommit([], result.value);
          }
        });
      }
    }

    this.setState({
      value: nextStructDateValue
    });
  }

  handlePrimaryInputCommit(path, value) {
    const initialValue = this.getValue(primaryFieldName);

    if ((value || initialValue) && value !== initialValue) {
      this.handleInputCommit([primaryFieldName], value);
    }
  }

  handlePrimaryInputChange(value) {
    this.setState({
      open: true,
      primaryValue: value
    });
  }

  renderParseFailedMessage() {
    const parseFailed = this.state.parseFailed;


    if (!parseFailed) {
      return null;
    }

    const formatParseFailedMessage = this.props.formatParseFailedMessage;


    return _react2.default.createElement(
      'div',
      { className: _Message2.default.warning },
      formatParseFailedMessage()
    );
  }

  render() {
    var _props2 = this.props;

    const formatFieldLabel = _props2.formatFieldLabel,
          formatOptionLabel = _props2.formatOptionLabel,
          optionLists = _props2.optionLists,
          terms = _props2.terms,
          readOnly = _props2.readOnly,
          defaultValue = _props2.defaultValue,
          formatParseFailedMessage = _props2.formatParseFailedMessage,
          name = _props2.name,
          parentPath = _props2.parentPath,
          subpath = _props2.subpath,
          valueProp = _props2.value,
          onCommit = _props2.onCommit,
          onMount = _props2.onMount,
          parseDisplayDate = _props2.parseDisplayDate,
          remainingProps = _objectWithoutProperties(_props2, ['formatFieldLabel', 'formatOptionLabel', 'optionLists', 'terms', 'readOnly', 'defaultValue', 'formatParseFailedMessage', 'name', 'parentPath', 'subpath', 'value', 'onCommit', 'onMount', 'parseDisplayDate']);

    var _state = this.state;
    const open = _state.open,
          primaryValue = _state.primaryValue,
          value = _state.value;


    return _react2.default.createElement(
      DropdownInput,
      _extends({}, remainingProps, {
        className: readOnly ? _StructuredDateInput2.default.readOnly : _StructuredDateInput2.default.normal,
        commitUnchanged: true,
        open: open,
        value: primaryValue,
        readOnly: readOnly,
        isOpenableWhenReadOnly: true,
        onChange: this.handlePrimaryInputChange,
        onClose: this.handleDropdownInputClose,
        onCommit: this.handlePrimaryInputCommit,
        onOpen: this.handleDropdownInputOpen
      }),
      _react2.default.createElement(
        _CustomCompoundInput2.default,
        { readOnly: readOnly, value: value },
        _react2.default.createElement(
          _Row2.default,
          null,
          _react2.default.createElement(
            'div',
            null,
            _react2.default.createElement(LabelableTextInput, {
              name: 'datePeriod',
              label: formatFieldLabel('datePeriod'),
              onCommit: this.handleInputCommit
            })
          ),
          _react2.default.createElement(
            'div',
            null,
            _react2.default.createElement(LabelableTextInput, {
              name: 'dateAssociation',
              label: formatFieldLabel('dateAssociation'),
              onCommit: this.handleInputCommit
            })
          ),
          _react2.default.createElement(
            'div',
            null,
            _react2.default.createElement(LabelableTextInput, {
              name: 'dateNote',
              label: formatFieldLabel('dateNote'),
              onCommit: this.handleInputCommit
            })
          )
        ),
        this.renderParseFailedMessage(),
        _react2.default.createElement(
          'table',
          null,
          _react2.default.createElement(
            'thead',
            null,
            _react2.default.createElement(
              'tr',
              null,
              _react2.default.createElement('th', null),
              _react2.default.createElement(
                'th',
                null,
                _react2.default.createElement(
                  _Label2.default,
                  null,
                  formatFieldLabel('dateYear')
                )
              ),
              _react2.default.createElement(
                'th',
                null,
                _react2.default.createElement(
                  _Label2.default,
                  null,
                  formatFieldLabel('dateMonth')
                )
              ),
              _react2.default.createElement(
                'th',
                null,
                _react2.default.createElement(
                  _Label2.default,
                  null,
                  formatFieldLabel('dateDay')
                )
              ),
              _react2.default.createElement(
                'th',
                null,
                _react2.default.createElement(
                  _Label2.default,
                  null,
                  formatFieldLabel('dateEra')
                )
              ),
              _react2.default.createElement(
                'th',
                null,
                _react2.default.createElement(
                  _Label2.default,
                  null,
                  formatFieldLabel('dateCertainty')
                )
              ),
              _react2.default.createElement(
                'th',
                null,
                _react2.default.createElement(
                  _Label2.default,
                  null,
                  formatFieldLabel('dateQualifier')
                )
              ),
              _react2.default.createElement(
                'th',
                null,
                _react2.default.createElement(
                  _Label2.default,
                  null,
                  formatFieldLabel('dateQualifierValue')
                )
              ),
              _react2.default.createElement(
                'th',
                null,
                _react2.default.createElement(
                  _Label2.default,
                  null,
                  formatFieldLabel('dateQualifierUnit')
                )
              )
            )
          ),
          _react2.default.createElement(
            'tbody',
            null,
            _react2.default.createElement(
              'tr',
              null,
              _react2.default.createElement(
                'th',
                null,
                _react2.default.createElement(
                  _Label2.default,
                  null,
                  formatFieldLabel('earliestSingle')
                )
              ),
              _react2.default.createElement(
                'td',
                null,
                _react2.default.createElement(TextInput, {
                  embedded: true,
                  name: 'dateEarliestSingleYear',
                  onCommit: this.handleInputCommit
                })
              ),
              _react2.default.createElement(
                'td',
                null,
                _react2.default.createElement(TextInput, {
                  embedded: true,
                  name: 'dateEarliestSingleMonth',
                  onCommit: this.handleInputCommit
                })
              ),
              _react2.default.createElement(
                'td',
                null,
                _react2.default.createElement(TextInput, {
                  embedded: true,
                  name: 'dateEarliestSingleDay',
                  onCommit: this.handleInputCommit
                })
              ),
              _react2.default.createElement(
                'td',
                null,
                _react2.default.createElement(_TermPickerInput2.default, {
                  embedded: true,
                  name: 'dateEarliestSingleEra',
                  terms: terms.dateera,
                  onCommit: this.handleInputCommit
                })
              ),
              _react2.default.createElement(
                'td',
                null,
                _react2.default.createElement(_TermPickerInput2.default, {
                  embedded: true,
                  name: 'dateEarliestSingleCertainty',
                  terms: terms.datecertainty,
                  onCommit: this.handleInputCommit
                })
              ),
              _react2.default.createElement(
                'td',
                null,
                _react2.default.createElement(SubstringFilteringDropdownMenuInput, {
                  embedded: true,
                  formatOptionLabel: formatOptionLabel,
                  name: 'dateEarliestSingleQualifier',
                  options: optionLists.dateQualifiers,
                  onCommit: this.handleInputCommit
                })
              ),
              _react2.default.createElement(
                'td',
                null,
                _react2.default.createElement(TextInput, {
                  embedded: true,
                  name: 'dateEarliestSingleQualifierValue',
                  onCommit: this.handleInputCommit
                })
              ),
              _react2.default.createElement(
                'td',
                null,
                _react2.default.createElement(_TermPickerInput2.default, {
                  embedded: true,
                  name: 'dateEarliestSingleQualifierUnit',
                  terms: terms.datequalifier,
                  onCommit: this.handleInputCommit
                })
              )
            ),
            _react2.default.createElement(
              'tr',
              null,
              _react2.default.createElement(
                'th',
                null,
                _react2.default.createElement(
                  _Label2.default,
                  null,
                  formatFieldLabel('latest')
                )
              ),
              _react2.default.createElement(
                'td',
                null,
                _react2.default.createElement(TextInput, {
                  embedded: true,
                  name: 'dateLatestYear',
                  value: this.getValue('dateLatestYear'),
                  onCommit: this.handleInputCommit
                })
              ),
              _react2.default.createElement(
                'td',
                null,
                _react2.default.createElement(TextInput, {
                  embedded: true,
                  name: 'dateLatestMonth',
                  value: this.getValue('dateLatestMonth'),
                  onCommit: this.handleInputCommit
                })
              ),
              _react2.default.createElement(
                'td',
                null,
                _react2.default.createElement(TextInput, {
                  embedded: true,
                  name: 'dateLatestDay',
                  value: this.getValue('dateLatestDay'),
                  onCommit: this.handleInputCommit
                })
              ),
              _react2.default.createElement(
                'td',
                null,
                _react2.default.createElement(_TermPickerInput2.default, {
                  embedded: true,
                  name: 'dateLatestEra',
                  terms: terms.dateera,
                  value: this.getValue('dateLatestEra'),
                  onCommit: this.handleInputCommit
                })
              ),
              _react2.default.createElement(
                'td',
                null,
                _react2.default.createElement(_TermPickerInput2.default, {
                  embedded: true,
                  name: 'dateLatestCertainty',
                  terms: terms.datecertainty,
                  value: this.getValue('dateLatestCertainty'),
                  onCommit: this.handleInputCommit
                })
              ),
              _react2.default.createElement(
                'td',
                null,
                _react2.default.createElement(SubstringFilteringDropdownMenuInput, {
                  embedded: true,
                  formatOptionLabel: formatOptionLabel,
                  name: 'dateLatestQualifier',
                  options: optionLists.dateQualifiers,
                  value: this.getValue('dateLatestQualifier'),
                  onCommit: this.handleInputCommit
                })
              ),
              _react2.default.createElement(
                'td',
                null,
                _react2.default.createElement(TextInput, {
                  embedded: true,
                  name: 'dateLatestQualifierValue',
                  value: this.getValue('dateLatestQualifierValue'),
                  onCommit: this.handleInputCommit
                })
              ),
              _react2.default.createElement(
                'td',
                null,
                _react2.default.createElement(_TermPickerInput2.default, {
                  embedded: true,
                  name: 'dateLatestQualifierUnit',
                  terms: terms.datequalifier,
                  value: this.getValue('dateLatestQualifierUnit'),
                  onCommit: this.handleInputCommit
                })
              )
            )
          )
        )
      )
    );
  }
}

exports.default = StructuredDateInput;
StructuredDateInput.propTypes = propTypes;
StructuredDateInput.defaultProps = defaultProps;