'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _TextInput = require('./TextInput');

var _TextInput2 = _interopRequireDefault(_TextInput);

var _DropdownMenuInput = require('./DropdownMenuInput');

var _DropdownMenuInput2 = _interopRequireDefault(_DropdownMenuInput);

var _FileInput = require('./FileInput');

var _FileInput2 = _interopRequireDefault(_FileInput);

var _changeable = require('../enhancers/changeable');

var _changeable2 = _interopRequireDefault(_changeable);

var _committable = require('../enhancers/committable');

var _committable2 = _interopRequireDefault(_committable);

var _labelable = require('../enhancers/labelable');

var _labelable2 = _interopRequireDefault(_labelable);

var _pathHelpers = require('../helpers/pathHelpers');

var _UploadInput = require('../../styles/cspace-input/UploadInput.css');

var _UploadInput2 = _interopRequireDefault(_UploadInput);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* global File */

const DropdownMenuInput = (0, _labelable2.default)(_DropdownMenuInput2.default);
const TextInput = (0, _labelable2.default)((0, _committable2.default)((0, _changeable2.default)(_TextInput2.default)));
const FileInput = (0, _labelable2.default)(_FileInput2.default);

const propTypes = {
  /* eslint-disable react/no-unused-prop-types */
  name: _propTypes2.default.string,
  parentPath: _pathHelpers.pathPropType,
  subpath: _pathHelpers.pathPropType,
  /* eslint-enable react/no-unused-prop-types */
  type: _propTypes2.default.string,
  value: _propTypes2.default.oneOfType([_propTypes2.default.arrayOf(_propTypes2.default.instanceOf(File)), _propTypes2.default.string]),
  readOnly: _propTypes2.default.bool,
  typeInputLabel: _propTypes2.default.string,
  fileOptionLabel: _propTypes2.default.string,
  urlOptionLabel: _propTypes2.default.string,
  fileInputLabel: _propTypes2.default.string,
  fileChooseButtonLabel: _propTypes2.default.string,
  urlInputLabel: _propTypes2.default.string,
  formatFileInfo: _propTypes2.default.func,
  onCommit: _propTypes2.default.func,
  onTypeChanged: _propTypes2.default.func
};

const defaultProps = {
  type: 'file',
  typeInputLabel: 'Upload',
  fileOptionLabel: 'local file',
  urlOptionLabel: 'external media',
  fileInputLabel: 'File',
  fileChooseButtonLabel: undefined,
  urlInputLabel: 'URL',
  formatFileInfo: undefined
};

class UploadInput extends _react.Component {
  constructor(props) {
    super(props);

    this.handleTypeInputCommit = this.handleTypeInputCommit.bind(this);
    this.handleFileInputCommit = this.handleFileInputCommit.bind(this);
    this.handleUrlInputCommit = this.handleUrlInputCommit.bind(this);

    this.state = {
      type: props.type
    };
  }

  componentWillReceiveProps(nextProps) {
    this.setState({
      type: nextProps.type
    });
  }

  handleTypeInputCommit(path, value) {
    const prevValue = this.state.value;

    this.setState({
      type: value
    });

    var _props = this.props;
    const onCommit = _props.onCommit,
          onTypeChanged = _props.onTypeChanged;


    if (onCommit) {
      onCommit((0, _pathHelpers.getPath)(this.props), null);
    }

    if (onTypeChanged && value !== prevValue) {
      onTypeChanged(value);
    }
  }

  handleFileInputCommit(path, value) {
    const onCommit = this.props.onCommit;


    if (onCommit) {
      onCommit((0, _pathHelpers.getPath)(this.props), value);
    }
  }

  handleUrlInputCommit(path, value) {
    const onCommit = this.props.onCommit;


    if (onCommit) {
      onCommit((0, _pathHelpers.getPath)(this.props), value);
    }
  }

  renderTypeInput() {
    var _props2 = this.props;
    const readOnly = _props2.readOnly,
          typeInputLabel = _props2.typeInputLabel,
          fileOptionLabel = _props2.fileOptionLabel,
          urlOptionLabel = _props2.urlOptionLabel;
    const type = this.state.type;


    const options = [{ value: 'file', label: fileOptionLabel }, { value: 'url', label: urlOptionLabel }];

    return _react2.default.createElement(DropdownMenuInput, {
      label: typeInputLabel,
      name: 'type',
      readOnly: readOnly,
      options: options,
      value: type,
      onCommit: this.handleTypeInputCommit
    });
  }

  renderFileInput() {
    var _props3 = this.props;
    const readOnly = _props3.readOnly,
          value = _props3.value,
          fileInputLabel = _props3.fileInputLabel,
          fileChooseButtonLabel = _props3.fileChooseButtonLabel,
          formatFileInfo = _props3.formatFileInfo;


    const fileList = value instanceof Array ? value : null;

    return _react2.default.createElement(FileInput, {
      label: fileInputLabel,
      name: 'file',
      readOnly: readOnly,
      value: fileList,
      chooseButtonLabel: fileChooseButtonLabel,
      formatFileInfo: formatFileInfo,
      onCommit: this.handleFileInputCommit
    });
  }

  renderUrlInput() {
    var _props4 = this.props;
    const readOnly = _props4.readOnly,
          value = _props4.value,
          urlInputLabel = _props4.urlInputLabel;


    const externalUrl = typeof value === 'string' ? value : null;

    return _react2.default.createElement(TextInput, {
      label: urlInputLabel,
      name: 'url',
      readOnly: readOnly,
      value: externalUrl,
      onCommit: this.handleUrlInputCommit
    });
  }

  render() {
    const type = this.state.type;


    const fileOrUrlInput = type === 'file' ? this.renderFileInput() : this.renderUrlInput();

    return _react2.default.createElement(
      'div',
      { className: _UploadInput2.default.common },
      this.renderTypeInput(),
      fileOrUrlInput
    );
  }
}

exports.default = UploadInput;
UploadInput.propTypes = propTypes;
UploadInput.defaultProps = defaultProps;