'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = VocabularyInput;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _SubstringFilteringDropdownMenuInput = require('./SubstringFilteringDropdownMenuInput');

var _SubstringFilteringDropdownMenuInput2 = _interopRequireDefault(_SubstringFilteringDropdownMenuInput);

var _withLabeledOptions = require('../enhancers/withLabeledOptions');

var _withLabeledOptions2 = _interopRequireDefault(_withLabeledOptions);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

const SubstringFilteringDropdownMenuInput = (0, _withLabeledOptions2.default)(_SubstringFilteringDropdownMenuInput2.default);

const propTypes = {
  formatVocabularyLabel: _propTypes2.default.func,
  recordTypes: _propTypes2.default.object,
  recordType: _propTypes2.default.string,
  rootVocabulary: _propTypes2.default.string,
  value: _propTypes2.default.string
};

const defaultProps = {
  formatVocabularyLabel: (name, config) => {
    const messages = config.messages;

    if (messages) {
      return messages.name.defaultMessage;
    }

    return name;
  },
  rootVocabulary: 'all'
};

function VocabularyInput(props) {
  const formatVocabularyLabel = props.formatVocabularyLabel,
        recordTypes = props.recordTypes,
        recordType = props.recordType,
        rootVocabulary = props.rootVocabulary,
        vocabularyValue = props.value,
        remainingProps = _objectWithoutProperties(props, ['formatVocabularyLabel', 'recordTypes', 'recordType', 'rootVocabulary', 'value']);

  if (!recordType || !recordTypes[recordType] || !recordTypes[recordType].vocabularies) {
    return null;
  }

  const vocabularies = recordTypes[recordType].vocabularies;
  const options = [];

  if (vocabularies[rootVocabulary]) {
    options.push({
      value: rootVocabulary,
      label: formatVocabularyLabel(rootVocabulary, vocabularies[rootVocabulary])
    });
  }

  const vocabularyNames = Object.keys(vocabularies).filter(vocabularyName => vocabularyName !== rootVocabulary && !vocabularies[vocabularyName].disabled).sort((nameA, nameB) => {
    const configA = vocabularies[nameA];
    const configB = vocabularies[nameB];

    // Primary sort by sortOrder

    let sortOrderA = configA.sortOrder;
    let sortOrderB = configB.sortOrder;

    if (typeof sortOrderA !== 'number') {
      sortOrderA = Number.MAX_VALUE;
    }

    if (typeof sortOrderB !== 'number') {
      sortOrderB = Number.MAX_VALUE;
    }

    if (sortOrderA !== sortOrderB) {
      return sortOrderA > sortOrderB ? 1 : -1;
    }

    // Secondary sort by label

    const labelA = formatVocabularyLabel(nameA, configA);
    const labelB = formatVocabularyLabel(nameB, configB);

    // FIXME: This should be locale aware
    return labelA.localeCompare(labelB);
  });

  vocabularyNames.forEach(vocabularyName => {
    options.push({
      indent: 1,
      value: vocabularyName,
      label: formatVocabularyLabel(vocabularyName, vocabularies[vocabularyName])
    });
  });

  let value = vocabularyValue;

  if (!value) {
    for (let i = 0; i < vocabularyNames.length; i += 1) {
      const vocabularyName = vocabularyNames[i];
      const vocabulary = vocabularies[vocabularyName];

      if (vocabulary.defaultForSearch) {
        value = vocabularyName;
        break;
      }
    }

    if (!value && options.length > 0) {
      value = options[0].value;
    }
  }

  return _react2.default.createElement(SubstringFilteringDropdownMenuInput, _extends({
    blankable: false,
    options: options,
    value: value
  }, remainingProps));
}

VocabularyInput.propTypes = propTypes;
VocabularyInput.defaultProps = defaultProps;