'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = changeable;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

/**
 * An adapter for onChange events. Returns an enhanced component that handles onChange events fired
 * by the base component, and executes a callback with the new value.
 * @param {string|function} BaseComponent - The component to enhance. This component must accept an
 * onChange prop.
 * @returns {function} The enhanced component, which accepts an onChange prop containing a function
 * to be called with the new value.
 */
function changeable(BaseComponent) {
  const baseComponentName = BaseComponent.displayName || BaseComponent.name || 'Component';

  const propTypes = _extends({}, BaseComponent.propTypes, {
    autoSyncValue: _propTypes2.default.bool,
    onChange: _propTypes2.default.func,
    value: _propTypes2.default.string
  });

  const defaultProps = {
    autoSyncValue: true,
    value: ''
  };

  class Changeable extends _react.Component {
    constructor(props) {
      super(props);

      this.state = {
        value: props.value
      };

      this.handleChange = this.handleChange.bind(this);
    }

    componentWillReceiveProps(nextProps) {
      this.setState({
        value: nextProps.value
      });
    }

    handleChange(event) {
      const value = event.target.value;

      var _props = this.props;
      const onChange = _props.onChange,
            autoSyncValue = _props.autoSyncValue;


      if (autoSyncValue) {
        this.setState({
          value
        });
      }

      if (onChange) {
        onChange(value);
      }
    }

    render() {
      var _props2 = this.props;

      const autoSyncValue = _props2.autoSyncValue,
            onChange = _props2.onChange,
            remainingProps = _objectWithoutProperties(_props2, ['autoSyncValue', 'onChange']);

      const value = this.state.value;


      return _react2.default.createElement(BaseComponent, _extends({}, remainingProps, {
        value: value,
        onChange: this.handleChange
      }));
    }
  }

  Changeable.propTypes = propTypes;
  Changeable.defaultProps = defaultProps;
  Changeable.displayName = `changeable(${baseComponentName})`;

  return Changeable;
}