'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = committable;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _pathHelpers = require('../helpers/pathHelpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

/**
 * Returns an enhanced component that detects that a change to the value of the base component has
 * been committed, and executes a callback with the new value. A change is considered to be
 * committed when the base component loses focus, or the enter key is pressed while it has focus.
 * @param {string|function} BaseComponent - The component to enhance. This component must accept
 * onBlur and onKeyPress props.
 * @returns {function} The enhanced component, which accepts an onCommit prop containing a function
 * to be called with the path to the input, and its new value.
 */
function committable(BaseComponent) {
  const baseComponentName = BaseComponent.displayName || BaseComponent.name || 'Component';

  const propTypes = _extends({}, BaseComponent.propTypes, {
    commitUnchanged: _propTypes2.default.bool,
    readOnly: _propTypes2.default.bool,
    onBlur: _propTypes2.default.func,
    onCommit: _propTypes2.default.func,
    onKeyPress: _propTypes2.default.func
  });

  const defaultProps = {
    commitUnchanged: false
  };

  class Committable extends _react.Component {
    constructor(props) {
      super(props);

      this.handleBlur = this.handleBlur.bind(this);
      this.handleKeyPress = this.handleKeyPress.bind(this);
    }

    handleBlur(event) {
      var _props = this.props;
      const onBlur = _props.onBlur,
            readOnly = _props.readOnly;


      if (onBlur) {
        onBlur(event);
      }

      if (!readOnly) {
        this.commit(event.target.value);
      }
    }

    handleKeyPress(event) {
      if (event.key === 'Enter') {
        this.commit(event.target.value);
      }

      const onKeyPress = this.props.onKeyPress;


      if (onKeyPress) {
        onKeyPress(event);
      }
    }

    commit(value) {
      var _props2 = this.props;
      const commitUnchanged = _props2.commitUnchanged,
            initialValue = _props2.value,
            onCommit = _props2.onCommit;


      if (onCommit && ((value || initialValue) && value !== initialValue || commitUnchanged)) {
        onCommit((0, _pathHelpers.getPath)(this.props), value);
      }
    }

    render() {
      var _props3 = this.props;

      const commitUnchanged = _props3.commitUnchanged,
            onCommit = _props3.onCommit,
            onKeyPress = _props3.onKeyPress,
            remainingProps = _objectWithoutProperties(_props3, ['commitUnchanged', 'onCommit', 'onKeyPress']);

      return _react2.default.createElement(BaseComponent, _extends({}, remainingProps, {
        onBlur: this.handleBlur,
        onKeyPress: this.handleKeyPress
      }));
    }
  }

  Committable.propTypes = propTypes;
  Committable.defaultProps = defaultProps;
  Committable.displayName = `committable(${baseComponentName})`;

  return Committable;
}